#!/usr/bin/env perl -w

# @(#) MetaData.pm 1.1.0 03/02/12 18:49:30
# @(#) 

use strict;
use English;
use Data::Dumper;
use DBI;
use XML::Simple;

my @ISA;
my @EXPORT;
my @EXPORT_OK;

BEGIN {
  $WM::MetaData::VERSION = "1.1.0";
}

{
  package WM::MetaData;
  
  #  use DynaLoader ();
  use Exporter ();
  use English;

  my $VERSION;

  my $conf;

  BEGIN {
    #@ISA = qw(Exporter DynaLoader);
    @ISA = qw(Exporter);
    
    # package export info
    @EXPORT    = qw( readFile readDB compare writeXML );
    @EXPORT_OK = qw( ); 
    
    #bootstrap MetaData;
  }
  
  return 1;


  sub readConf {
    #my $class = shift;

    unless ( defined $conf and ref $conf ) {
      $conf = [ { desc => "Data Dictionary",
		  key => "DD",
		  query => "select field_name, field_type from wmd_dict_field" 
		}
	      ]
    }
  }


  sub readFile {
    my $class = shift;
    my $configFile = shift;

    my $self = {};
    bless $self;

    readConf;

    my $fType = `file $configFile`;
    if ( $fType =~ m/gzip compressed data/ ) {
      open DATAFILE, "gzip -d -c <$configFile |" or die "Can't open input file $configFile\n";
    }
    else {
      open DATAFILE, "<$configFile" or die "Can't open input file $configFile\n";
    }

    print "Reading stored configuration from $configFile \n";

    while ( <DATAFILE> ) {
      chomp;
    }
    
    print "Completed reading stored configuration.\n";

    close DATAFILE;
    
    return $self;
  }


  sub readDB {
    my $class = shift;
    my $connectString = shift;

    my $self = {};
    bless $self;

    readConf;

    my $dbName = $connectString;
    $dbName =~ s/\/.*\@/\@/;
    
    $self->{ input } = { type => 'db', 
			 connect => $connectString,
			 name => $dbName
		       };
    $self->{ data } = { };

    my %indexList;
    
    my $dbh = DBI->connect( 'dbi:Oracle:', $connectString, '', { PrintError => 0, ora_module_name => "SchemaInf.pm" } )  
      or die "Error: Can't connect to '$connectString' ($DBI::errstr).\n";
    
    print "Reading database configuration from $dbName .. \n";

    foreach my $mdRef ( @{ $conf } ) {
      my $mdDesc = $mdRef->{ desc };
      printf "  Reading %s\n", $mdDesc;
      my $statement = $mdRef->{ query };
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}
	$self->{ data }->{ $mdRef->{ key } } = []  unless exists $self->{ data }->{ $mdRef->{ key } };
	push @{ $self->{ data }->{ $mdRef->{ key } } }, $returnRowRef;
      }
    }      
    
    return $self;
  }


  sub hashRef {
    my $self = shift;
    
#    my $temp = { test=> { a => 1, b => 2, c => [ 5, 6, 7 ] } };
#    return $temp;
    return $self;
  }


  sub writeXML {
    my $self = shift;
    my $configFile = shift;
    #my $attrRef = shift;

    #die 'Usage: $h->writeFile( $fileName [,\%attr])'  unless defined $configFile and $configFile and ( not $attrRef or ( ref $attrRef and ( ref $attrRef eq 'HASH' ) ) );

    # Initialize attributes so we don't have to check them too carefully later.
#    $attrRef = { } unless defined $attrRef;
#    $attrRef->{ ZipFile } = 0  unless exists $attrRef->{ ZipFile };
#    if ( $attrRef->{ Zipfile }) {
#      open DATAFILE, "| gzip -c >$configFile" or die "Can't open output file $configFile\n";
#    } else {
#      open DATAFILE, ">$configFile" or die "Can't open output file $configFile\n";
#    }

#    XML::Simple::XMLout( $self->{ data }, rootname => 'MetaData', outputfile => $configFile );
    XML::Simple::XMLout( $self->{ data }, rootname => 'MetaData', outputfile => $configFile );

    #close DATAFILE;
#    print "INC= ", join("\n", @INC ), "\n";
#    open DATAFILE, ">dump.out";
#    $Data::Dumper::Indent = 3;
#    print DATAFILE Data::Dumper->Dump( [ $self ] );
#    close DATAFILE;
  }


  sub writeFile {
    my $self = shift;
    my $configFile = shift;
    my $attrRef = shift;

    die 'Usage: $h->writeFile( $fileName [,\%attr])'  unless defined $configFile and $configFile and ( not $attrRef or ( ref $attrRef and ( ref $attrRef eq 'HASH' ) ) );

    # Initialize attributes so we don't have to check them too carefully later.
    $attrRef = { } unless defined $attrRef;
    $attrRef->{ ZipFile } = 0  unless exists $attrRef->{ ZipFile };
    if ( $attrRef->{ Zipfile }) {
      open DATAFILE, "| gzip -c >$configFile" or die "Can't open output file $configFile\n";
    } else {
      open DATAFILE, ">$configFile" or die "Can't open output file $configFile\n";
    }

    print "Writing stored configuration to $configFile \n";

    my $hostName = `hostname`;
    chomp $hostName;
    my $domainName = `domainname`;
    chomp $domainName;
    my $runDate = `date`;
    chomp $runDate;
    print  DATAFILE "#\tMetaData.pm 1.1.0 03/02/12 18:49:30\n";
    printf DATAFILE "#\tcreated: %s\n", $runDate;
    printf DATAFILE "#\thost = %s, domain = %s\n", $hostName, $domainName;
    printf DATAFILE "#\tsource = '%s' %s\n", $self->{ input }->{ type }, $self->{ input }->{ name };

    foreach my $qryKey ( keys %{ $self->{ data } } ) {
      printf DATAFILE "KEY\t%s\n", $qryKey;
      foreach my $rowRef ( @{ $self->{ data }->{ $qryKey } } ) {
	my $first = 1;
	foreach my $colName ( keys %{ $rowRef } ) {
	  print DATAFILE "\t"  unless $first;
	  $first = 0;
	  printf DATAFILE "%s=\"%s\"", $colName, $rowRef->{ $colName };
	}
	printf DATAFILE "\n";
      }
    }
    close DATAFILE;

    print "Completed writing stored configuration.\n";
  }


  sub compare {

  }

}

