#!/usr/bin/env perl -w

# @(#) Compare.pm 1.1.0 03/02/12 18:49:30
# @(#) 

use strict;
use English;
use DBI;

my @ISA;
my @EXPORT;
my @EXPORT_OK;

BEGIN {
  $SchemaInf::Compare::VERSION = "1.1.0";
}

{
  package SchemaInf::Compare;

  #  use DynaLoader ();
  use Exporter ();
  use English;

  my $VERSION;

  BEGIN {
    @ISA = qw(Exporter);
    
    # package export info
    @EXPORT    = qw( compareSchemas, showDifferences );
    @EXPORT_OK = qw( compareSchemas, showDifferences ); 
  }
  
  return 1;


  sub _printMessage( $$@ );
  sub _compareTable( $$$ );

  sub alterWrite {
    return 1
  }

  sub compareSequence {
    return 1
  }

  sub compareSchemas {
    my $class = shift;
    my $sourceSchema = shift;
    my $destSchema = shift;
    my $callAttrs = shift;

    # Check the inputs, make sure everybody is as expected.
    die "SchemaInf::Compare::compareSchema -> sourceSchema is not a reference\n"  unless ref( $sourceSchema );
    die "SchemaInf::Compare::compareSchema -> sourceSchema(" . ref( $sourceSchema) . ") not correct type\n" unless  ref( $sourceSchema ) eq "SchemaInf";
    die "SchemaInf::Compare::compareSchema -> destSchema is not a reference\n"  unless ref( $destSchema );
    die "SchemaInf::Compare::compareSchema -> destSchema(" . ref( $destSchema) . ") not correct type\n" unless ref( $destSchema ) eq "SchemaInf";
    $callAttrs = { }  unless defined $callAttrs and ref $callAttrs and ref $callAttrs eq "HASH";

    my $self = { };
    bless $self;

    my $self{ diff } = { source => $sourceSchema->{ input }->{ name },
			 dest   => $destSchema->{ input }->{ name }
		       };

    my $showProgress = exists $callAttrs{ showProgress } and defined $callAttrs{ showProgress } and $callAttrs{ showProgress };

    printf "Comparing %s configuration to %s configuration...\n", $self->{ diff }->{ source }, $self->{ diff }->{ source }      if $showProgress;

    printf "   Checking for Missing table info\n"   if $showProgress;
    foreach my $tabName ( sort keys %{ $sourceSchema->{ tables } } ) {
      if ( not exists $destSchema->{ tables }->{ $tabName } ) {
	$self->{ tables }->{ $tabName } = $sourceSchema->{ tables }->{ $tabName};
	$self->{ tables }->{ $tabName }->{ MISSING } = 1;
      }
      else {
	_compareTable( $sourceSchema, $destSchema, $tabName );
      }
    }

    printf "   Checking for Extra table info\n"   if $showProgress;
    foreach my $tabName ( sort keys %{ $destSchema->{ tables } } ) {
      if ( not exists $sourceSchema->{ tables }->{ $tabName } ) {
	$self->{ tables }->{ $tabName }->{ EXTRA } = 1;
      }
      else {
	foreach my $colName ( sort keys %{ $destSchema->{ tables }->{ $tabName }->{ columns } } ) {
	  if ( not exists $sourceSchema->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
	    $self->{ tables }->{ $tabName }->{ columns }->{  $colName }->{ EXTRA } = 1;
	  }
	}
	foreach my $indName ( sort keys %{ $destSchema->{ tables }->{ $tabName }->{ indexes } } ) {
	  if ( not exists $sourceSchema->{ tables }->{ $tabName }->{ indexes }->{ $indName } ) {
	    $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ EXTRA } = 1;
	  }
	}
	foreach my $consName ( sort keys %{ $destSchema->{ tables }->{ $tabName }->{ constraints } } ) {
	  if ( not exists $sourceSchema->{ tables }->{ $tabName }->{ constraints }->{ $consName } ) {
	    my $redundant = 0;
	    my $srch = $destSchema->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ search };
	    if ( defined $srch and $srch =~ m/^\s*(\w+)\s+IS\s+NOT\s+NULL\s*$/ ) {
	      my $colName = $1;
	      if ( exists $sourceSchema->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
		$redundant = ( $sourceSchema->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ nullable } eq 'N' );
	      }
	    }
	    if ( $redundant ) {
	      $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ REDUNDANT } = 1;
	    }
	    else {
	      $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ EXTRA } = 1;
	    }
	  }
	}
      }
    }

    printf "   Checking for Missing sequence info\n"   if $showProgress;
    foreach my $seqName ( sort keys %{ $sourceSchema->{ sequences } } ) {
      if ( not exists $destSchema->{ sequences }->{ $seqName } ) {
	$self->{ sequences }->{ $seqName } = $sourceSchema->{ sequences };
	$self->{ sequences }->{ $seqName }->{ MISSING } = 1;
      }
      else {
	compareSequence( $sourceSchema, $destSchema, $seqName );
      }
    }

    printf "   Checking for Extra sequence info\n"   if $showProgress;
    foreach my $seqName ( sort keys %{ $destSchema->{ sequences } } ) {
      if ( not exists $sourceSchema->{ sequences }->{ $seqName } ) {
	$self->{ sequences }->{ $seqName }->{ EXTRA } = 1;
      }
    }

  }


  sub showDifferences {
    my $self = shift;

    my $sourceName = $self->{ diff }->{ source };
    my $destName = $self->{ diff }->{ dest };

    foreach my $tabName ( sort keys %{ $self->{ tables } } ) {
      if ( exists $self->{ tables }->{ $tabName }->{ MISSING } ) {
	_printMessage(3, "Table %s in %s is missing in %s.", $tabName, $sourceName, $destName );
      }
      else {
	#compareTable( $sourceSchema, $destSchema, $tabName );
      }
    }
 
    foreach my $tabName ( sort keys %{ $self->{ tables } } ) {
      if ( exists $self->{ tables }->{ $tabName }->{ EXTRA } ) {
	_printMessage(1, "Table %s is extra in %s.", $tabName, $destName );
      }
      else {
	foreach my $colName ( sort keys %{ $self->{ tables }->{ $tabName }->{ columns } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ columns }->{  $colName }->{ EXTRA } ) {
	    _printMessage(1, "Column %s.%s is extra in %s.", $tabName, $colName, $destName );
	  }
	}
	foreach my $indName ( sort keys %{ $self->{ tables }->{ $tabName }->{ indexes } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ EXTRA } ) {
	    _printMessage(2, "Index %s on table %s is extra in %s.", $indName, $tabName, $destName );
	  }
	}
	foreach my $consName ( sort keys %{ $self->{ tables }->{ $tabName }->{ constraints } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ EXTRA } ) {
	      _printMessage(2, "Constraint %s on table %s is extra in %s.", $consName, $tabName, $destName );
	  }
	  elsif ( exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ REDUNDANT } ) {
	      _printMessage(1, "Constraint %s on table %s is extra in %s but equivalent to 'NOT NULL' column", $consName, $tabName, $destName );
	  }
	}
      }
    }

    foreach my $seqName ( sort keys %{ $sourceSchema->{ sequences } } ) {
      if ( exists $self->{ sequences }->{ $seqName }->{ MISSING } ) {
	_printMessage(3, "Sequence %s in %s is missing in %s.", $seqName, $sourceName, $destName );
      }
      else {
	#compareSequence( $sourceSchema, $destSchema, $seqName );
      }
    }

    foreach my $seqName ( sort keys %{ $destSchema->{ sequences } } ) {
      if ( exists $self->{ sequences }->{ $seqName }->{ EXTRA } ) {
	_printMessage( 1, "Sequence %s is extra in %s.", $seqName, $destName );
      }
    }

  }
}




__END__

  {
    my ( $srcConfig, $dstConfig );
    my ( $tabName, $colName );
    my ( $srcName, $dstName );
    
    sub compareTabColAttr( $;$ ) {
      my $attrName = shift;
      my $chkCons = shift;
      
      if ( exists $srcConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName } and exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName } ) {
	my $srcAttr = $srcConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName };
	my $dstAttr = $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName };
	if ( defined $srcAttr ) {
	  if ( not defined $dstAttr ) {
	    printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	    alterWrite('ALTER', 'COLUMN_C', $srcConfig, $tabName, $colName, $attrName ) if defined $writeFix;
	  }
	  elsif ( $srcAttr ne $dstAttr ) {
	    my $matchingCons = 0;
	    if ( $chkCons ) {
	      if ( $attrName eq 'nullable' ) {
		my $tempConst = sprintf("%s IS NOT NULL", $colName);
		if ( $srcAttr eq 'Y' ) {
		  foreach my $dstCons ( sort keys %{ $dstConfig->{ tables }->{ $tabName }->{ constraints } } ) {
		    if ( $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstCons }->{ search } eq $tempConst ) {
		      $matchingCons = 1;
		      last;
		    }
		  }
		}
		else {
		  foreach my $srcCons ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ constraints } } ) {
		    if ( $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $srcCons }->{ search } eq $tempConst ) {
		      $matchingCons = 1;
		      last;
		    }
		  }
		}
	      }
	    }
	    if ( not $matchingCons ) {
	      printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	      alterWrite('ALTER', 'COLUMN', $srcConfig, $tabName, $colName, $attrName ) if defined $writeFix;
	    }
	    else {
	      printMessage(1, "Column %s.%s - %s mismatch: %s => %s and %s => %s - Handled by Constraint", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	    }
	  }
	}
	elsif ( defined $dstAttr ) {
	  printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
	}
      }
    }


    sub _compareTableColumn ( $ ) {
      $colName = shift;
      
      if ( not exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
      printMessage(3, "Column %s.%s in %s is missing in %s.", $tabName, $colName, $srcName, $dstName );
      alterWrite('ADD', 'COLUMN', $srcConfig, $tabName, $colName ) if defined $writeFix;
    }
    else {
      compareTabColAttr( 'type' );
      compareTabColAttr( 'len' );
      compareTabColAttr( 'prec' );
      compareTabColAttr( 'scale' );
      compareTabColAttr( 'nullable', 1 );
    }
  }


  sub _compareTabAttr( $;$ ) {
    my $attrName = shift;
    my $sectionName = shift;
    
    my $srcTabRef = $srcConfig->{ tables }->{ $tabName };
    my $dstTabRef = $dstConfig->{ tables }->{ $tabName };
    if ( defined $sectionName ) {
      $srcTabRef = $srcTabRef->{ $sectionName };
      $dstTabRef = $dstTabRef->{ $sectionName };
    }

    if ( exists $srcTabRef->{ $attrName } and exists $dstTabRef->{ $attrName } ) {
      my $srcAttr = $srcTabRef->{ $attrName };
      my $dstAttr = $dstTabRef->{ $attrName };

      $srcAttr = join( ":", @{ $srcAttr } )  if ( defined $srcAttr and ref $srcAttr eq 'ARRAY' );
      $dstAttr = join( ":", @{ $dstAttr } )  if ( defined $dstAttr and ref $dstAttr eq 'ARRAY' );

      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  if ( $srcAttr ne '' ) {
	    printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	    alterWrite('ALTER', 'TABLE_C', $srcConfig, $tabName, $attrName ) if defined $writeFix;
	  }
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  alterWrite('ALTER', 'TABLE', $srcConfig, $tabName, $attrName ) if defined $writeFix;
	}
      }
      elsif ( defined $dstAttr and $dstAttr ne '' ) {
	printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub _compareTable( $$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };

    if ( exists $srcConfig->{ tables }->{ $tabName }->{ type } and exists $dstConfig->{ tables }->{ $tabName }->{ type } ) {
      my $srcType = $srcConfig->{ tables }->{ $tabName }->{ type };
      my $dstType = $dstConfig->{ tables }->{ $tabName }->{ type };
      if (  $srcType ne  $dstType ) {
	printMessage( 3, "Table/View %s mismatch: %s is %s and %s is %s", $tabName, $srcName, $srcType, $dstName, $dstType );
	if ( defined $writeFix ) {
	  if ( $srcType eq 'TABLE' ) {
	    alterWrite('DROP', 'VIEW', $tabName );
	    alterWrite('CREATE', 'TABLE', $srcConfig, $tabName );
	  }
	  else {
	    alterWrite('DROP', 'TABLE', $tabName );
	    alterWrite('CREATE', 'VIEW', $srcConfig, $tabName );
	  }
	}
      }
    }

    # Check table attributes
    compareTabAttr( 'tablespace' );
    compareTabAttr( 'monitoring' );
    compareTabAttr( 'partitioned' );
    compareTabAttr( 'iot' );
    compareTabAttr( 'temp' );
    compareTabAttr( 'pct_free' );
    #compareTabAttr( 'pct_used' );
    compareTabAttr( 'freelists' );
    compareTabAttr( 'freelist_groups' );
    compareTabAttr( 'duration' );

    compareTabAttr( 'tablespace', 'part_info' );
    compareTabAttr( 'part_type', 'part_info' );
    compareTabAttr( 'part_key_count', 'part_info' );
    compareTabAttr( 'pct_free', 'part_info' );
    compareTabAttr( 'pct_used', 'part_info' );
    compareTabAttr( 'freelists', 'part_info' );
    compareTabAttr( 'freelist_groups', 'part_info' );
    compareTabAttr( 'part_keys', 'part_info' );
    
    # Check table columns
    foreach my $colName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ columns } } ) {
      _compareTableColumn( $colName );
    }
    
    # Check Table indexes
    foreach my $indName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ indexes } } ) {
      my $srcIndRef = $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };
      if ( not exists $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName } ) {
	# Check to see if we can find this index with another name
	my $indMatch = 0;
	my $dstIndName;
	foreach my $tIndName ( keys %{ $dstConfig->{ tables }->{ $tabName }->{ indexes } } ) {
	  $dstIndName = $tIndName;
	  my $dstIndRef = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $tIndName };
	  $indMatch = checkIndex( $srcConfig, $dstConfig, $tabName, $indName, $tIndName );
	  last if $indMatch;
	}
	if ( $indMatch ) {
	  printMessage(0, "Index %s on table %s in %s is renamed to %s in %s.", $indName, $tabName, $srcName, $dstIndName, $dstName );
	  alterWrite('RENAME', 'INDEX', $srcConfig, $tabName, $indName, $dstIndName ) if defined $writeFix and not (  $dstIndName =~ m/^SYS_C/ or $indName =~ m/^SYS_C/ );
	  $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName } = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName };
	  delete $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName };
	  compareIndex( $srcConfig, $dstConfig, $tabName, $indName );
	}
	else {
	  printMessage(3, "Index %s on table %s in %s is missing in %s.", $indName, $tabName, $srcName, $dstName );
	  alterWrite('CREATE', 'INDEX', $srcConfig, $tabName, $indName ) if defined $writeFix;
	}
      }
      else {
	compareIndex( $srcConfig, $dstConfig, $tabName, $indName );
      }
    }

    # Check Table Constraints
    foreach my $consName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ constraints } } ) {
      my $srcConsRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      if ( not exists $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName } ) {
	# Check to see if we can find this constraint with another name
	my $consMatch = 0;
	my $dstConsName;
	foreach my $tConsName ( keys %{ $dstConfig->{ tables }->{ $tabName }->{ constraints } } ) {
	  $dstConsName = $tConsName;
	  my $dstConsRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $tConsName };
	  $consMatch = checkConstraint( $srcConfig, $dstConfig, $tabName, $consName, $tConsName );
	  last if $consMatch;
	}
	if ( $consMatch ) {
	  printMessage(0, "Constraint %s on table %s in %s is renamed to %s in %s.", $consName, $tabName, $srcName, $dstConsName, $dstName );
	  alterWrite('RENAME', 'CONSTRAINT', $srcConfig, $tabName, $consName, $dstConsName ) if defined $writeFix and not (  $dstConsName =~ m/^SYS_C/ or $consName =~ m/^SYS_C/ );
	  $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName } = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };
	  delete $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };
	  compareConstraint( $srcConfig, $dstConfig, $tabName, $consName );
	}
	else {
	  my $redundant = 0;
	  my $srch = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ search };
	  if ( defined $srch and $srch =~ m/^\s*(\w+)\s+IS\s+NOT\s+NULL\s*$/ ) {
	    my $colName = $1;
	    if ( exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
	      $redundant = ( $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ nullable } eq 'N' );
	    }
	  }
	  if ( $redundant ) {
	    printMessage(1, "Constraint %s on table %s in %s is missing in %s but handled by 'NOT NULL' column", $consName, $tabName, $srcName, $dstName );
	    alterWrite('ADD', 'CONSTRAINT_NN', $srcConfig, $tabName, $consName ) if defined $writeFix;
	  }
	  else {
	    printMessage(2, "Constraint %s on table %s in %s is missing in %s.", $consName, $tabName, $srcName, $dstName );
	    alterWrite('ADD', 'CONSTRAINT', $srcConfig, $tabName, $consName ) if defined $writeFix;
	  }
	}
      }
      else {
	compareConstraint( $srcConfig, $dstConfig, $tabName, $consName );
      }
    }
  }

}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcSeqRef, $dstSeqRef );
  my $seqName;


  sub compareSeqAttr( $ ) {
    my $attrName = shift;

    if ( exists $srcSeqRef->{ $attrName } and exists $dstSeqRef->{ $attrName } ) {
      my $srcAttr = $srcSeqRef->{ $attrName };
      my $dstAttr = $dstSeqRef->{ $attrName };
      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	  alterWrite('ALTER', 'SEQUENCE', $srcConfig, $seqName ) if defined $writeFix;
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  alterWrite('ALTER', 'SEQUENCE', $srcConfig, $seqName ) if defined $writeFix;
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareSequence( $$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $seqName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcSeqRef = $srcConfig->{ sequences }->{ $seqName };
    $dstSeqRef = $dstConfig->{ sequences }->{ $seqName };

    compareSeqAttr( 'cycleFlag' );
    compareSeqAttr( 'orderFlag' );
  }
}




sub checkIndex( $$$$$ ) {
  my $srcConfig = shift;
  my $dstConfig = shift;
  my $tabName = shift;
  my $srcIndName = shift;
  my $dstIndName = shift;

  my $result = 1;

  my $saveIndCols = join(":", @{  $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $srcIndName }->{ columns } });
  my $dbIndCols = join(":", @{ $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName }->{ columns } });

  $result = 0 if ( $saveIndCols ne $dbIndCols );

  return $result;
}


sub checkConstraint( $$$$$ ) {
  my $srcConfig = shift;
  my $dstConfig = shift;
  my $tabName = shift;
  my $srcConsName = shift;
  my $dstConsName = shift;

  my $srcConsRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $srcConsName };
  my $dstConsRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };

  my $result = 1;

  #print "Check Constraint: $tabName.$srcConsName with $tabName.$dstConsName\n";
  #print "    Types:  src = '", $srcConsRef->{ type },"',  dst = '",$dstConsRef->{ type },"'\n";
  $result = 0 if ( $srcConsRef->{ type } ne $dstConsRef->{ type } );
  if ( exists $srcConsRef->{ search } and exists $dstConsRef->{ search } ) {
    my $cmprSrc = $srcConsRef->{ search };
    my $cmprDst = $dstConsRef->{ search };
    $cmprSrc =~ s/\s//g;
    $cmprDst =~ s/\s//g;
    $result = 0 if ( $cmprSrc ne $cmprDst );
  }

  my $saveConsCols = join(":", sort { $srcConsRef->{ columns }->{ $a }->{ position } <=> $srcConsRef->{ columns }->{ $b }->{ position } } keys %{  $srcConsRef->{ columns } });
  my $dbConsCols = join(":", sort { $dstConsRef->{ columns }->{ $a }->{ position } <=> $dstConsRef->{ columns }->{ $b }->{ position } } keys %{ $dstConsRef->{ columns } });

  $result = 0 if ( $saveConsCols ne $dbConsCols );

  #print "    Result = $result\n";
  return $result;
}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcIndRef, $dstIndRef );
  my $tabName;
  my $indName;


  sub compareIndexAttr( $;$ ) {
    my $attrName = shift;
    my $sectionName = shift;

    my $srcRef = $srcIndRef;
    my $dstRef = $dstIndRef;

    $srcRef = $srcRef->{ $sectionName }  if defined $sectionName;
    $dstRef = $dstRef->{ $sectionName }  if defined $sectionName;

    if ( exists $srcRef->{ $attrName } and exists $dstRef->{ $attrName } ) {
      my $srcAttr = $srcRef->{ $attrName };
      my $dstAttr = $dstRef->{ $attrName };

      $srcAttr = join( ":", @{ $srcAttr } )  if ( defined $srcAttr and ref $srcAttr eq 'ARRAY' );
      $dstAttr = join( ":", @{ $dstAttr } )  if ( defined $dstAttr and ref $dstAttr eq 'ARRAY' );

      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
 	  alterWrite('ALTER', 'INDEX', $srcConfig, $tabName, $indName, $attrName ) if defined $writeFix;
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
 	  alterWrite('ALTER', 'INDEX', $srcConfig, $tabName, $indName, $attrName ) if defined $writeFix;
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareIndex( $$$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;
    $indName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcIndRef = $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };
    $dstIndRef = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };

    compareIndexAttr( 'type' );
    compareIndexAttr( 'uniqueness' );
    compareIndexAttr( 'tablespace' );
    compareIndexAttr( 'compression' );
    compareIndexAttr( 'pct_free' );
    compareIndexAttr( 'freelists' );
    compareIndexAttr( 'freelist_groups' );

    compareIndexAttr( 'tablespace', 'part_info' );
    compareIndexAttr( 'part_type', 'part_info' );
    compareIndexAttr( 'part_key_count', 'part_info' );
    compareIndexAttr( 'pct_free', 'part_info' );
    compareIndexAttr( 'freelists', 'part_info' );
    compareIndexAttr( 'freelist_groups', 'part_info' );
    compareIndexAttr( 'part_keys', 'part_info' );

    my $srcIndCols = join(":", @{ $srcIndRef->{ columns } });
    my $dstIndCols = join(":", @{ $dstIndRef->{ columns } });
    if ( $srcIndCols ne $dstIndCols ) {
      printMessage( 3, "Index %s on table %s, col def '%s' in %s differs from '%s' in %s.\n", $indName, $tabName, $dstIndCols, $dstName, $srcIndCols, $srcName );
      alterWrite('REBUILD', 'INDEX', $srcConfig, $tabName, $indName ) if defined $writeFix;
    }
  }

}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcConRef, $dstConRef );
  my $tabName;
  my $consName;


  sub compareConstraintAttr( $;$ ) {
    my $attrName = shift;
    my $compress = shift;

    if ( exists $srcConRef->{ $attrName } and exists $dstConRef->{ $attrName } ) {
      my $srcAttr = $srcConRef->{ $attrName };
      my $dstAttr = $dstConRef->{ $attrName };
      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
 	  alterWrite('ALTER', 'CONSTRAINT', $srcConfig, $tabName, $consName, $attrName ) if defined $writeFix;
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  my $diff = 1;
	  if ( defined $compress ) {
	    my $cSrcAttr = $srcAttr;
	    my $cDstAttr = $dstAttr;
	    $cSrcAttr =~ s/\s//g;
	    $cDstAttr =~ s/\s//g;
	    $diff = ( $cSrcAttr ne $cDstAttr );
	  }
	  if ( $diff and ( length( $srcAttr ) > 30 or length( $dstAttr ) > 30 ) ) {
	    printMessage(2, "Constraint %s.%s - %s mismatch:\n\t%30s => %s\n\t%30s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	    alterWrite('ALTER', 'CONSTRAINT', $srcConfig, $tabName, $consName, $attrName ) if defined $writeFix;
	  }
	  elsif ( $diff ) {
	    printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  }
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
	alterWrite('ALTER', 'CONSTRAINT', $srcConfig, $tabName, $consName, $attrName ) if defined $writeFix;
      }
    }
  }


  sub compareConstraint( $$$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;
    $consName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcConRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };
    $dstConRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };

    compareConstraintAttr( 'type' );
    compareConstraintAttr( 'status' );
    compareConstraintAttr( 'refName' );
    compareConstraintAttr( 'delRule' );
    compareConstraintAttr( 'deferrable' );
    compareConstraintAttr( 'deferred' );
    compareConstraintAttr( 'search', 1 );

    my $srcConCols = join(":", sort { $srcConRef->{ columns }->{ $a }->{ position } <=> $srcConRef->{ columns }->{ $b }->{ position } } keys %{ $srcConRef->{ columns } });
    my $dstConCols = join(":", sort { $dstConRef->{ columns }->{ $a }->{ position } <=> $dstConRef->{ columns }->{ $b }->{ position } } keys %{ $dstConRef->{ columns } });
    if ( $srcConCols ne $dstConCols ) {
      printMessage( 3, "Constraint %s on table %s, col def '%s' in %s differs from '%s' in %s.\n", $consName, $tabName, $dstConCols, $dstName, $srcConCols, $srcName );
      alterWrite('REBUILD', 'CONSTRAINT', $srcConfig, $tabName, $consName ) if defined $writeFix;
    }
  }

}


sub printMessage( $$@ ) {
  my $level = shift;
  my $msg = shift;
  my @params = @ARG;
  
  my @labels = ( "NOTE:", "INFO:", "WARNING:", "ERROR:" );

  $level = 0 if ( $level < 0 );
  $level = $#labels if ( $level > $#labels );

  ( $level >= $verbosity ) and printf "  %-8s $msg\n", $labels[ $level ], @params;
}
