#!/usr/bin/env perl -w

# @(#) Repair.pm 1.1.0 03/02/12 18:49:31
# @(#) 

use strict;
use English;
use DBI;

my @ISA;
my @EXPORT;
my @EXPORT_OK;

BEGIN {
  $SchemaInf::Repair::VERSION = "1.1.0";
}

{
  package SchemaInf::Repair;

  #  use DynaLoader ();
  use Exporter ();
  use English;

  my $VERSION;

  BEGIN {
    @ISA = qw(Exporter, SchemaInf::Compare);
    
    # package export info
    @EXPORT    = qw( writeUpdates );
    @EXPORT_OK = qw( writeUpdates ); 
  }
  
  return 1;


  sub alterWrite {
    return 1;
  }


  sub writeUpdates {
    my $self = shift;
    my $writeFix = shift;

    my $sourceName = $self->{ diff }->{ source };
    my $destName = $self->{ diff }->{ dest };

    open FIXFILE, ">$writeFix" or die "Couldn't open $writeFix for output.\n";

    foreach my $tabName ( sort keys %{ $self->{ tables } } ) {
      if ( exists $self->{ tables }->{ $tabName }->{ MISSING } ) {
	alterWrite('CREATE', uc $self->{ tables }->{ $tabName}->{ type }, $self, $tabName );
      }
      else {
	#compareTable( $sourceSchema, $destSchema, $tabName );
      }
    }
    
    foreach my $tabName ( sort keys %{ $self->{ tables } } ) {
      if ( exists $self->{ tables }->{ $tabName }->{ EXTRA } ) {
	alterWrite('DROP', 'TABLE_C', $self, $tabName );
      }
      else {
	foreach my $colName ( sort keys %{ $self->{ tables }->{ $tabName }->{ columns } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ columns }->{  $colName }->{ EXTRA } ) {
	    alterWrite('DROP', 'COLUMN', $self, $tabName, $colName );
	  }
	}
	foreach my $indName ( sort keys %{ $self->{ tables }->{ $tabName }->{ indexes } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ EXTRA } ) {
	    alterWrite('DROP', 'INDEX_C', $self, $tabName, $indName );
	  }
	}
	foreach my $consName ( sort keys %{ $self->{ tables }->{ $tabName }->{ constraints } } ) {
	  if ( exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ EXTRA } ) {
	    alterWrite('DROP', 'CONSTRAINT', $sourceSchema, $tabName, $consName );
	  }
	  elsif ( exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ REDUNDANT } ) {
	    # Redundant constraint - rename?
	  }
	}
      }
    }

    foreach my $seqName ( sort keys %{ $sourceSchema->{ sequences } } ) {
      if ( exists $self->{ sequences }->{ $seqName }->{ MISSING } ) {
	alterWrite('CREATE', 'SEQUENCE', $sourceSchema, $seqName );
      }
      else {
	#compareSequence( $sourceSchema, $destSchema, $seqName );
      }
    }

    foreach my $seqName ( sort keys %{ $destSchema->{ sequences } } ) {
      if ( exists $self->{ sequences }->{ $seqName }->{ EXTRA } ) {
	alterWrite('DROP', 'SEQUENCE', $sourceSchema, $seqName );
      }
    }

    close FIXFILE;
  }


__END__

sub createSequence( $$ ) {
  my $config = shift;
  my $seqName = shift;

  my $seqRef = $config->{ sequences }->{ $seqName };
  printf FIXFILE "--\n--  New Sequence %s\n--\n", $seqName;
  printf FIXFILE "CREATE SEQUENCE %s", $seqName;
  printf FIXFILE " CYCLE"  if ( exists $seqRef->{ cycleFlag } and defined $seqRef->{ cycleFlag } and $seqRef->{ cycleFlag } eq 'Y' );
  printf FIXFILE " ORDER"  if ( exists $seqRef->{ orderFlag } and defined $seqRef->{ orderFlag } and $seqRef->{ orderFlag } eq 'Y' );
  print  FIXFILE ";\n\n\n";
#    printf DATAFILE "SEQUENCE\t%s\t%s\t%s\n", $seqName, $seqRef->{ cycleFlag }, $seqRef->{ orderFlag };
}


sub createIndex( $$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $indName = shift;

  my $tabRef = $config->{ tables }->{ $tabName };
  my $indRef = $tabRef->{ indexes }->{ $indName };

  printf FIXFILE "--\n--  New Index %s.%s\n--\n", $tabName, $indName;
  printf FIXFILE "CREATE";
  printf FIXFILE " UNIQUE"  if ( exists $indRef->{ uniqueness } and defined $indRef->{ uniqueness } and $indRef->{ uniqueness } eq 'UNIQUE' );
  printf FIXFILE " INDEX %s ON %s ( %s )", $indName, $tabName, join(", ", @{ $indRef->{ columns } } );
  printf FIXFILE "\nCOMPRESS %i", $#{ $indRef->{ columns } }   if exists $indRef->{ compression } and defined $indRef->{ compression } and $indRef->{ compression } eq 'ENABLED';
  my $physRef = $indRef;
  $physRef = $indRef->{ part_info }   if exists $indRef->{ part_info };
  printf FIXFILE "\nPCTFREE %i", $physRef->{ pct_free } if ( defined $physRef->{ pct_free } and $physRef->{ pct_free } ne '' );
  printf FIXFILE "\nTABLESPACE %s", $physRef->{ tablespace } if ( defined $physRef->{ tablespace } and $physRef->{ tablespace } ne '' );
  printf FIXFILE "\nSTORAGE (\n  FREELISTS %i\n)", $physRef->{ freelists } if ( defined $physRef->{ freelists } and $physRef->{ freelists } ne '' );
  print  FIXFILE ";\n\n\n"
}


sub consColList( $ ) {
  my $consRef = shift;
  return sort { $consRef->{ columns }->{ $a }->{ position } <=> $consRef->{ columns }->{ $b }->{ position } } keys %{  $consRef->{ columns } };
}


sub createConstraint( $$$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $consName = shift;
  my $mode = shift;

  my $tabRef = $config->{ tables }->{ $tabName };
  my $consRef = $tabRef->{ constraints }->{ $consName };

  printf FIXFILE " CONSTRAINT"  unless ( $mode eq 'column' and $consName =~ m/^SYS_C/ );
  printf FIXFILE " %s", $consName  unless ( $consName =~ m/^SYS_C/ );

  my $consType = $consRef->{ type };
  if ( $consType eq 'C' ) {
    printf FIXFILE " CHECK (%s)", $consRef->{ search };
  }
  elsif ( $consType eq 'P' ) {
    printf FIXFILE " PRIMARY KEY";
    if ( $mode ne 'column' ) {
      printf FIXFILE " ( %s )", join(", ", consColList( $consRef ) );
    }
  }
  elsif ( $consType eq 'R' ) {
    my $refConsName = $consRef->{ refName };
    my $refConsTabName = $config->{ constraints }->{ $refConsName }->{ table };
    my $refConsRef = $config->{ tables }->{ $refConsTabName }->{ constraints }->{ $refConsName };
    if ( $mode ne 'column' ) {
      printf FIXFILE " FOREIGN KEY ( %s )", join(", ", consColList( $consRef ) );
    }
    printf FIXFILE " REFERENCES %s", $refConsTabName;
    if ( $refConsRef->{ type } ne 'P' ) {
      printf FIXFILE " ( %s )", join(", ", consColList( $refConsRef ) );
    }
    else {
      if ( $consRef->{ delRule } ne 'NO ACTION' ) {
	printf FIXFILE " ON DELETE %s", $consRef->{ delRule };
      }
    }
  }
  elsif ( $consType eq 'U' ) {
    printf FIXFILE " UNIQUE";
    if ( $mode ne 'column' ) {
      printf FIXFILE " ( %s )", join(", ", consColList( $consRef ) );
    }
  }
  elsif ( $consType eq 'O' ) {
    die "Can't handle 'O' constraint.\n";
  }
}


sub createColumn( $$$;\% ) {
  my $config = shift;
  my $tabName = shift;
  my $colName = shift;
  my $builtRef = shift;
  
  my $tabRef = $config->{ tables }->{ $tabName };
  my $colRef = $tabRef->{ columns }->{ $colName };

  printf FIXFILE " %-20s", $colName;
  my $colType = uc $colRef->{ type };
  if ( $colType =~ m/^(N?VARCHAR2|N?CHAR|RAW)$/ ) {
    # column types CHAR, NCHAR, VARCHAR2, NVARCHAR2, RAW
    printf FIXFILE " %s(%i)", $colType, $colRef->{ len };
  }
  elsif ( $colType =~ m/^(LONG|DATE|ROWID)$/ ) {
    # column types DATE, LONG and ROWID
    printf FIXFILE " %s", $colType;
  }
  elsif ( $colType eq 'FLOAT' ) {
    # column type FLOAT
    printf FIXFILE " %s(%i)", $colType, $colRef->{ prec };
  }
  elsif ( $colType eq 'NUMBER' ) {
    # column type NUMBER
    if ( $colRef->{ prec } eq "" ) {
      printf FIXFILE " %s", $colType;
    }
    elsif ( $colRef->{ scale } eq "" ) {
      printf FIXFILE " %s(%i)", $colType, $colRef->{ prec };
    }
    else {
      printf FIXFILE " %s(%i, %i)", $colType, $colRef->{ prec }, $colRef->{ scale };
    }
  }
  else {
    # object types, e.g. GENERICMESSAGE
    printf FIXFILE " %s", $colType;
  }

  # Default value for column
  if ( defined $colRef->{ default } and $colRef->{ default } ne '' ) {
    printf FIXFILE " DEFAULT %s", $colRef->{ default };
  }

  if ( $colRef->{ nullable } eq 'N' and exists $colRef->{ notNull } ) {
    # Need a NOT NULL constraint - otherwise should come from PK
    $builtRef->{ $colRef->{ notNull } } = 1   if defined $builtRef;
    printf FIXFILE " NOT NULL";
  }

  # get the column level constraints
  foreach my $consName ( @{ $colRef->{ constraints } } ) {
    # make sure not already handled (e.g. NOT NULL)
    unless ( defined $builtRef and exists $builtRef->{ $consName } ) {
      my $consRef = $tabRef->{ constraints }->{ $consName };
      # make sure its a 'single column' constraint
      if ( $#{ [ keys %{ $consRef->{ columns } } ] } == 0 ) {
	createConstraint( $config, $tabName, $consName, 'column' );
	$builtRef->{ $consName } = 1   if defined $builtRef;
      }
    }
  }
}


sub createTable( $$ ) {
  my $config = shift;
  my $tabName = shift;

  my $tabRef = $config->{ tables }->{ $tabName };
  my %builtCons = ( );
  printf FIXFILE "--\n--  New Table %s\n--\n", $tabName;
  print  FIXFILE "CREATE";
  print  FIXFILE " GLOBAL TEMPORARY" if ( defined $tabRef->{ temp } and $tabRef->{ temp } eq 'Y' );
  printf FIXFILE " TABLE %s\n", $tabName;
  printf FIXFILE "(";
  my $first = 1;
  foreach my $colName ( sort { $tabRef->{ columns }->{ $a }->{ ordinal } <=> $tabRef->{ columns }->{ $b }->{ ordinal } } keys %{ $tabRef->{ columns } } ) {
    my $colRef = $tabRef->{ columns }->{ $colName };
    printf FIXFILE ",\n " unless $first;
    $first = 0;

    createColumn( $config, $tabName, $colName, %builtCons );
  }

  ### Need table level constraints next.
  foreach my $consName ( keys %{ $tabRef->{ constraints } } ) {
    # make sure not already handled
    unless ( exists $builtCons{ $consName } ) {
      printf FIXFILE ",\n " unless $first;
      $first = 0;
      my $consRef = $tabRef->{ constraints }->{ $consName };
      createConstraint( $config, $tabName, $consName, 'table' );
      $builtCons{ $consName } = 1;
    }
  }
  printf FIXFILE "\n)";

  my $physRef = $tabRef;
  my $iotName;
  if ( defined $tabRef->{ iot } and $tabRef->{ iot } eq 'IOT' ) {
    printf FIXFILE "\nORGANIZATION INDEX";
    $iotName = $tabRef->{ iot_index };
    $physRef = $tabRef->{ indexes }->{ $iotName } if defined $iotName;
    printf FIXFILE " COMPRESS %i", $#{ $physRef->{ columns } }   if ( exists $physRef->{ compression } and defined $physRef->{ compression } and $physRef->{ compression } eq 'ENABLED' );
  }
  if ( defined $tabRef->{ partitioned } and $tabRef->{ partitioned } eq 'YES' ) {
    my $partRef = $tabRef->{ part_info };
    $physRef = $physRef->{ part_info } if exists $physRef->{ part_info };
    printf FIXFILE "\nPARTITION BY %s (%s)\n(\n", $partRef->{ part_type }, join( ', ', @{ $partRef->{ part_keys } } );
    printf FIXFILE "  PARTITION x VALUES LESS THAN (z)\n)";
  }
  printf FIXFILE "\nMONITORING"  if ( defined $tabRef->{ monitoring } and $tabRef->{ monitoring } eq 'YES' );
  printf FIXFILE "\nPCTFREE %i", $physRef->{ pct_free } if ( defined $physRef->{ pct_free } and $physRef->{ pct_free } ne '' );
  printf FIXFILE "\nTABLESPACE %s", $physRef->{ tablespace } if ( defined $physRef->{ tablespace } and $physRef->{ tablespace } ne '' );
  printf FIXFILE "\nSTORAGE (\n  FREELISTS %i\n)", $physRef->{ freelists } if ( defined $physRef->{ freelists } and $physRef->{ freelists } ne '' );
  if ( defined $tabRef->{ temp } and $tabRef->{ temp } eq 'Y'  ) {
    if ( exists $tabRef->{ duration } and defined $tabRef->{ duration } ) {
      printf FIXFILE "\nON COMMIT %s ROWS", 'PRESERVE'  if  ( $tabRef->{ duration } eq 'SYS$SESSION' );
      printf FIXFILE "\nON COMMIT %s ROWS", 'DELETE'    if  ( $tabRef->{ duration } eq 'SYS$TRANSACTION' );
    }
  }
  printf FIXFILE ";\n\n\n";


  foreach my $indName ( sort keys %{ $tabRef->{ indexes } } ) {
    # Make sure index not built for constraint
    unless ( exists $builtCons{ $indName } ) {
      createIndex( $config, $tabName, $indName );
    }
  }
  
}


sub alterTable ( $$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $attrName = shift;

  my $tabRef = $config->{ tables }->{ $tabName };

  printf FIXFILE "--\n--  Alter Table %s, %s\n--\n", $tabName, $attrName;
  printf FIXFILE "ALTER TABLE %s", $tabName;
  my $physRef = $tabRef;
  if ( defined $tabRef->{ iot } and $tabRef->{ iot } eq 'IOT' ) {
    my $iotName = $tabRef->{ iot_index };
    $physRef = $tabRef->{ indexes }->{ $iotName } if defined $iotName;
  }
  if ( defined $tabRef->{ partitioned } and $tabRef->{ partitioned } eq 'YES' ) {
    $physRef = $physRef->{ part_info } if exists $physRef->{ part_info };
  }
  
  printf FIXFILE " MOVE";
  #printf FIXFILE " TABLESPACE %s", $physRef->{ tablespace } if ( defined $physRef->{ tablespace } and $physRef->{ tablespace } ne '' );
  printf FIXFILE " PCTFREE %i",  $physRef->{ pct_free }   if ( $attrName eq 'pct_free' );
  print  FIXFILE ";\n\n";
}


sub alterIndex ( $$$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $indName = shift;
  my $attrName = shift;

  my $tabRef = $config->{ tables }->{ $tabName };
  my $indRef = $tabRef->{ indexes }->{ $indName };

  printf FIXFILE "--\n--  Alter Index %s.%s, %s\n--\n", $tabName, $indName, $attrName;
  printf FIXFILE "ALTER INDEX %s", $indName;
  my $physRef = $indRef;
  $physRef = $indRef->{ part_info }   if exists $indRef->{ part_info };
  
  printf FIXFILE " MOVE";
  #printf FIXFILE " MOVE TABLESPACE %s", $physRef->{ tablespace } if ( defined $physRef->{ tablespace } and $physRef->{ tablespace } ne '' );
  printf FIXFILE " PCTFREE %i",  $physRef->{ pct_free }   if ( $attrName eq 'pct_free' );
  print  FIXFILE ";\n\n";
}


sub addColumn( $$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $colName = shift;

  printf FIXFILE "--\n--  Add Column %s.%s\n--\n", $tabName, $colName;
  printf FIXFILE "ALTER TABLE %s ADD (", $tabName;
  createColumn( $config, $tabName, $colName );
  printf FIXFILE ");\n\n";
}


sub addConstraint( $$$ ) {
  my $config = shift;
  my $tabName = shift;
  my $consName = shift;

  printf FIXFILE "--\n--  Add Constraint %s on %s\n--\n", $consName, $tabName;
  printf FIXFILE "ALTER TABLE %s ADD (", $tabName;
  createConstraint( $config, $tabName, $consName, 'table' );
  printf FIXFILE ");\n\n";
}


sub alterWrite( @ ) {
  my $cmdType = uc shift;
  defined $cmdType or die "Undefined 'cmdType' to alterWrite.\n";

  my $cmdTarg = uc shift;
  defined $cmdTarg or die "Undefined 'cmdTarg' to alterWrite($cmdType).\n";

  my $config = shift;
  defined $config or die "Undefined 'config' to alterWrite($cmdType.$cmdTarg).\n";
  ref $config eq 'HASH' or die "'config' argument to alterWrite($cmdType.$cmdTarg) should be 'HASH', not '" . ref $config . "'\n";

  if ( $cmdType eq 'CREATE' ) {
    if ( $cmdTarg eq 'TABLE' ) {
      my $tabName = shift;
      defined $tabName or die "Undefined 'tabName' to alterWrite($cmdType.$cmdTarg).\n";
      createTable( $config, $tabName );
    }
    elsif ( $cmdTarg eq 'VIEW' ) {
      my $viewName = shift;
      defined $viewName or die "Undefined 'viewName' to alterWrite($cmdType.$cmdTarg).\n";
      #createView( $config, $viewName );
      printf FIXFILE "--\n-- Cannot create view %s (not implemented)\n--\n\n\n", $viewName;
    }
    elsif ( $cmdTarg eq 'INDEX' ) {
      my $tabName = shift;
      defined $tabName or die "Undefined 'tabName' to alterWrite($cmdType.$cmdTarg).\n";
      my $indName = shift;
      defined $indName or die "Undefined 'indName' to alterWrite($cmdType.$cmdTarg).\n";
      createIndex( $config, $tabName, $indName );
    }
    elsif ( $cmdTarg eq 'SEQUENCE' ) {
      my $seqName = shift;
      defined $seqName or die "Undefined 'seqName' to alterWrite($cmdType.$cmdTarg).\n";
      createSequence( $config, $seqName );
    }
    else {
      die "Unhandled 'cmdTarg' ($cmdTarg) to alterWrite(CREATE).\n";
    }
  }
  elsif ( $cmdType eq 'ALTER' ) {
    if ( $cmdTarg eq 'TABLE' ) {
      my $tabName = shift;
      my $attrName = shift;
    TABATTR:  {
	$attrName =~ m/pct_free/i  && do {
	  alterTable( $config, $tabName, $attrName );
	  last TABATTR;
	};
	printf FIXFILE " -- To do ALTER TABLE - %s => %s )\n", $tabName, $attrName;
      }
    }
    elsif ( $cmdTarg eq 'INDEX' ) {
      my $tabName = shift;
      my $indName = shift;
      my $attrName = shift;
    INDATTR:  {
	$attrName =~ m/pct_free/i  && do {
	  alterIndex( $config, $tabName, $indName, $attrName );
	  last INDATTR;
	};
	printf FIXFILE " -- To do ALTER INDEX - %s.%s => %s )\n", $tabName, $indName, $attrName;
      }
    }
    elsif ( $cmdTarg eq 'COLUMN' ) {
      my $tabName = shift;
      my $colName = shift;
      my $attrName = shift;
      printf FIXFILE " -- To do ALTER COLUMN - %s.%s => %s )\n", $tabName, $colName, $attrName;
    }
    else {
      printf FIXFILE " -- To do ALTER - %s ( %s )\n", $cmdTarg, join(', ', @_ );
    }
  }
  elsif ( $cmdType eq 'ADD' ) {
    if ( $cmdTarg eq 'COLUMN' ) {
      my $tabName = shift;
      my $colName = shift;
      defined $tabName or die "Undefined 'tabName' to alterWrite($cmdType.$cmdTarg).\n";
      defined $colName or die "Undefined 'colName' to alterWrite($cmdType.$cmdTarg).\n";
      addColumn( $config, $tabName, $colName );
    }
    elsif ( $cmdTarg eq 'CONSTRAINT' ) {
      my $tabName = shift;
      my $consName = shift;
      defined $tabName or die "Undefined 'tabName' to alterWrite($cmdType.$cmdTarg).\n";
      defined $consName or die "Undefined 'consName' to alterWrite($cmdType.$cmdTarg).\n";
      addConstraint( $config, $tabName, $consName );
    }
    else {
      die "Unhandled 'cmdTarg' ($cmdTarg) to alterWrite(CREATE).\n";
    }
  }
#  elsif ( $cmdType eq 'RENAME' ) {
#    if ( $cmdTarg eq 'INDEX' ) {
#      my $tabName = shift;
#      defined $tabName or die "Undefined 'tabName' to alterWrite($cmdType.$cmdTarg).\n";
#      createTable( $config, $tabName );
#    }
#    else {
#      die "Unhandled 'cmdTarg' ($cmdTarg) to alterWrite(CREATE).\n";
#    }
#  }
  else {
    die "Unhandled 'cmdType' ($cmdType) to alterWrite.\n";
  }
}


