package wm_xml_handler;

###################################################################
#        Copyright  2007 IBM Tivoli. All rights reserved         #
###################################################################
# Script   : wm_xml_handler                                       #
# Function : Read, merge, and process the core/modules            #
#            xml specifications and call out to all process-      #
#            ing tags for SBI Installations.                      #
###################################################################
# @(#) wm_xml_handler.pm ; 1.12.0 04/10/14 14:13:31

###################################################################
#              Standard/Supplied Perl Modules                     #
###################################################################
use strict;
use Env;
use XML::Twig;
###################################################################
#                      Watchmark Perl Modules                     #
###################################################################

###################################################################
#                        Global definitions                       #
###################################################################

my @Global_Pre_Validate;

my @Global_Pre_Install;

my @Global_Post_Install;

my @Global_Post_Validate;


# Arrays to handle multiple pre/post routines per module.

my @Modules_Pre_Validate;
my @Modules_Pre_Install;
my @Modules_Install;
my @Modules_Post_Install;
my @Modules_Post_Validate;

my $test_debug = "0";

my @Modules_Module;

###################################################################
#  Description of the Modules_Module Array of Hashes              #
###################################################################
my $Modules_Module_Structure = {
    SPEC_TYPE             => "",    # VENDOR or CORE
    NAME                  => "",
    PLATFORM              => "",
    VERSION               => "",
    TARFILE               => "",
    PROCESS_TYPE          => "",    # Values I:install U:Upgrade S:Skip
    MODULE_INSTALLDIR     => "",
    SPACE                 => "",
    PREREQ_CORE_PLATFORM  => "",
    PREREQ_CORE_VERSION   => "",
    PREREQ_MODULE_NAME    => "",
    PREREQ_MODULE_VERSION => ""
};

my $Module_Count = 0;

my $Core_Default_DBCONNECT       = "";
my $Core_Default_BASEDIR         = "";
my $Core_Default_CORE_INSTALLDIR = "";
my $Original_Core_INSTALLDIR     = "";
my $Core_Default_PORTBASE        = "";
my $Core_Default_LICENSEHOST     = "";
my $Core_Default_TMPDIR          = "";
my $Core_Default_CLASSPATH       = "";

my $Module_Default_DBCONNECT       = "";
my $Module_Default_BASEDIR         = "";
my $Module_Default_CORE_INSTALLDIR = "";
my $Module_Default_PORTBASE        = "";
my $Module_Default_LICENSEHOST     = "";
my $Module_Default_TMPDIR          = "";
my $Module_Default_CLASSPATH       = "";

my $Core_Module_NAME     = "";
my $Core_Module_PLATFORM = "";
my $Core_Module_VERSION  = "";
my $Core_Module_TARFILE  = "";
my $Core_Module_SPACE    = "";

my $Vendor_Module_INSTALLDIR = "";
my $Vendor_Module_NAME       = "";
my $Vendor_Module_VERSION    = "";
my $Vendor_Module_TARFILE    = "";
my $Vendor_Module_SPACE      = "";

my $Spec_Type = "";

my $spec_twig = new XML::Twig(
    TwigHandlers => {
        'DEFAULTS'                            => \&defaults,
        'MODULES/PRE_VALIDATE'                => \&global_pre_validate,
        'MODULES/PRE_INSTALL'                 => \&global_pre_install,
        'MODULES/POST_INSTALL'                => \&global_post_install,
        'MODULES/POST_VALIDATE'               => \&global_post_validate,
        'MODULES/MODULE/PREREQUISITES/CORE'   => \&core_prerequisites,
        'MODULES/MODULE/PREREQUISITES/MODULE' => \&module_prerequisites,
        'MODULES/MODULE/PRE_VALIDATE'         => \&modules_module_pre_validate,
        'MODULES/MODULE/PRE_INSTALL'          => \&modules_module_pre_install,
        'MODULES/MODULE/INSTALL'              => \&modules_module_install,
        'MODULES/MODULE/POST_VALIDATE'        => \&modules_module_post_validate,
        'MODULES/MODULE/POST_INSTALL'         => \&modules_module_post_install,
        'MODULES/MODULE'                      => \&modules_module
      }
);

###################################################################
#                    End Data Definitions                         #
###################################################################

sub defaults ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;

    #
    if ($test_debug) { print STDERR ">> defaults \n"; }

    #
    if ( $Spec_Type eq "CORE" ) {
        $Core_Default_DBCONNECT       = $name->first_child_text('DBCONNECT');
        $Core_Default_BASEDIR         = $name->first_child_text('BASEDIR');
        $Original_Core_INSTALLDIR     = $Core_Default_BASEDIR;
        $Core_Default_CORE_INSTALLDIR =
          $name->first_child_text('CORE_INSTALLDIR');
        $Original_Core_INSTALLDIR     = $Core_Default_CORE_INSTALLDIR;
        $Core_Default_PORTBASE        = $name->first_child_text('PORTBASE');
        $Core_Default_LICENSEHOST     = $name->first_child_text('LICENSEHOST');
        $Core_Default_TMPDIR          = $name->first_child_text('TMPDIR');
        $Core_Default_CLASSPATH       = $name->first_child_text('CLASSPATH');
    }
    else {
        $Module_Default_DBCONNECT       = $name->first_child_text('DBCONNECT');
        $Module_Default_BASEDIR         = $name->first_child_text('BASEDIR');
        $Module_Default_CORE_INSTALLDIR =
          $name->first_child_text('CORE_INSTALLDIR');
        $Module_Default_PORTBASE        = $name->first_child_text('PORTBASE');
        $Module_Default_LICENSEHOST     = $name->first_child_text('LICENSEHOST');
        $Module_Default_TMPDIR          = $name->first_child_text('TMPDIR');
        $Module_Default_CLASSPATH       = $name->first_child_text('CLASSPATH');
        
    }
}

sub modules_module ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;

    #
    if ($test_debug) { print STDERR ">> Entering modules_module\n"; }

    #
    if ( $Spec_Type eq "CORE" ) {
        $Modules_Module[$Module_Count]->{SPEC_TYPE} = "CORE";
        $Modules_Module[$Module_Count]->{NAME} = $Core_Module_NAME =
          $name->first_child_text('NAME');
        $Modules_Module[$Module_Count]->{PLATFORM} = $Core_Module_PLATFORM =
          $name->first_child_text('PLATFORM');
        $Modules_Module[$Module_Count]->{VERSION} = $Core_Module_VERSION =
          $name->first_child_text('VERSION');
        $Modules_Module[$Module_Count]->{TARFILE} = $Core_Module_TARFILE =
          $name->first_child_text('TARFILE');
        $Modules_Module[$Module_Count]->{SPACE} = $Core_Module_SPACE =
          $name->first_child_text('SPACE_MB');
    }
    else {
        $Modules_Module[$Module_Count]->{SPEC_TYPE} = "VENDOR";
        $Modules_Module[$Module_Count]->{NAME}      =
          $name->first_child_text('NAME');
        $Modules_Module[$Module_Count]->{PLATFORM} =
          $Modules_Module[$Module_Count]->{NAME};    # This is due to the fact that vendor's dont pass platforms
        $Modules_Module[$Module_Count]->{VERSION} =
          $name->first_child_text('VERSION');
        $Modules_Module[$Module_Count]->{TARFILE} =
          $name->first_child_text('TARFILE');
        $Modules_Module[$Module_Count]->{SPACE} =
          $name->first_child_text('SPACE_MB');
        $Modules_Module[$Module_Count]->{MODULE_INSTALLDIR} =
          $name->first_child_text('MODULE_INSTALLDIR');
    }

    ++$Module_Count;

}

sub modules_module_pre_validate ( $$ ) {
###################################################################
    #                                                                 #
###################################################################

    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) {
        print STDERR ">> Entering modules_module_pre_validate\n";
    }

    #

    push @Modules_Pre_Validate, join ( ":", $Module_Count, $value, $name-> {'att'} -> {'language'}  )
      unless $value eq "";

}

sub modules_module_pre_install ( $$ ) {
###################################################################
    #                                                                 #
###################################################################

    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) {
        print STDERR ">> Entering modules_module_pre_install\n";
    }

    #

    push @Modules_Pre_Install, join ( ":", $Module_Count, $value, $name-> {'att'} -> {'language'}  )
      unless $value eq "";

}

sub modules_module_install ( $$ ){
###################################################################
    #                                                                 #
###################################################################

    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) { print STDERR ">> Entering modules_module_install\n"; }

    #

    push @Modules_Install, join ( ":", $Module_Count, $value, $name-> {'att'} -> {'language'}  )
      unless $value eq "";

}

sub modules_module_post_install ( $$ ) {
###################################################################
    #                                                                 #
###################################################################

    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) {
        print STDERR ">> Entering modules_module_post_install\n";
    }

    #

    push @Modules_Post_Install, join ( ":", $Module_Count, $value, $name-> {'att'} -> {'language'}  )
      unless $value eq "";

}

sub modules_module_post_validate ( $$ ) {
###################################################################
    #                                                                 #
###################################################################

    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) {
        print STDERR ">> Entering modules_module_post_validate\n";
    }

    #

    push @Modules_Post_Validate, join ( ":", $Module_Count, $value, $name-> {'att'} -> {'language'}  )
      unless $value eq "";

}

sub core_prerequisites ( $$ ){
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;

    #
    if ($test_debug) { print STDERR ">> Entering core_prerequisites\n"; }

    #

    my ($platform) = $name->first_child_text('PLATFORM');
    my ($version)  = $name->first_child_text('VERSION');

    $Modules_Module[$Module_Count]->{PREREQ_CORE_PLATFORM} = $platform
      unless $platform eq "";
    $Modules_Module[$Module_Count]->{PREREQ_CORE_VERSION} = $version
      unless $version eq "";

}

sub module_prerequisites ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;

    #
    if ($test_debug) {
        print STDERR ">> Entering vendor_prerequisites_module\n";
    }

    #
    my ($version)  = $name->first_child_text('VERSION');
    my ($platform) = $name->first_child_text('PLATFORM');
    $Modules_Module[$Module_Count]->{PREREQ_MODULE_PLATFORM}    = $platform
      unless $platform eq "";
    $Modules_Module[$Module_Count]->{PREREQ_MODULE_VERSION} = $version
      unless $version eq "";

}

sub global_pre_validate ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) { print STDERR ">> Entering global_pre_validate\n"; }

    push @Global_Pre_Validate, (join ":",$Spec_Type, $value,$name-> {'att'} -> {'language'})  unless $value eq "";

    if ( $test_debug) { print STDERR ">> Global Pre Validate Routine - $Global_Pre_Validate[$#Global_Pre_Validate]\n"; }
}

sub global_pre_install ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) { print STDERR ">> Entering global_pre_install\n"; }

    #
    push @Global_Pre_Install, (join ":",$Spec_Type, $value,$name-> {'att'} -> {'language'})  unless $value eq "";
}

sub global_post_install ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;
    my $value = $name->text;

    #
    if ($test_debug) { print STDERR ">> Entering global_post_install\n"; }

    #

    push @Global_Post_Install, (join ":",$Spec_Type,$value,$name-> {'att'} -> {'language'})  unless $value eq "";
   

}

sub global_post_validate ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $twig, $name ) = @_;
    my $value = $name->text;


    #
    if ($test_debug) { print STDERR ">> Entering global_post_validate\n"; }

    #
    push @Global_Post_Validate, (join ":",$Spec_Type, $value,$name-> {'att'} -> {'language'})  unless $value eq "";


}

sub dump_modules_array ( ) {
###################################################################
    #                                                                 #
###################################################################

    #  print STDERR "Number of Modules ($#Modules_Module)\n";
    #
    if ($test_debug) { print STDERR ">> Entering dump_modules_array\n"; }

    #

    for ( my $x = 0 ; $x <= $#Modules_Module ; $x++ ) {

        print STDERR "Mudle Muddle Array -------------\n";
        print STDERR "Spec_TYPE      :  $Modules_Module[$x]->{SPEC_TYPE}\n";
        print STDERR "Name           :  $Modules_Module[$x]->{NAME}\n";
        print STDERR "Platform       :  $Modules_Module[$x]->{PLATFORM}\n";
        print STDERR "Version        :  $Modules_Module[$x]->{VERSION}\n";
        print STDERR "TARFILE        :  $Modules_Module[$x]->{TARFILE}\n";
        print STDERR
          "MOD_INSTALL_DIR:  $Modules_Module[$x]->{MODULE_INSTALLDIR}\n";
        print STDERR "SPACE          :  $Modules_Module[$x]->{SPACE}\n";
#        print STDERR "Core Platform  :  $Modules_Module[$x]->{CORE_PLATFORM}\n";
#        print STDERR "Core Version   :  $Modules_Module[$x]->{CORE_VERSION}\n";
#        print STDERR
#          "Module Version :  $Modules_Module[$x]->{MODULE_VERSION}\n";

        print STDERR "----------------------------------------\n";
    }

    print STDERR "\nGlobal Pre_Validates ---------------\n";
    for ( my $x = 0 ; $x <= $#Global_Pre_Validate ; $x++ ) {
        print STDERR " $x - $Global_Pre_Validate[$x]\n";
    }

    print STDERR "\nGlobal Pre_Install ---------------\n";
    for ( my $x = 0 ; $x <= $#Global_Pre_Install ; $x++ ) {
        print STDERR " $x - $Global_Pre_Install[$x]\n";
    }

    print STDERR "\nGlobal Post Install ---------------\n";
    for ( my $x = 0 ; $x <= $#Global_Post_Install ; $x++ ) {
        print STDERR " $x - $Global_Post_Install[$x]\n";
    }

    print STDERR "\nGlobal Post Validate ---------------\n";
    for ( my $x = 0 ; $x <= $#Global_Post_Validate ; $x++ ) {
        print STDERR " $x - $Global_Post_Validate[$x]\n";
    }

    print STDERR "\nModules Pre Validate ---------------\n";
    for ( my $x = 0 ; $x <= $#Modules_Pre_Validate ; $x++ ) {
        my ( $module_number, $routine ) =
          split ( ":", $Modules_Pre_Validate[$x] );
        print STDERR "$x - $module_number - $routine\n";
    }

    print STDERR "\nModules Pre Install ---------------\n";
    for ( my $x = 0 ; $x <= $#Modules_Pre_Install ; $x++ ) {
        my ( $module_number, $routine ) =
          split ( ":", $Modules_Pre_Install[$x] );
        print STDERR "$x - $module_number - $routine\n";
    }

    print STDERR "\nModules Install ---------------\n";
    for ( my $x = 0 ; $x <= $#Modules_Install ; $x++ ) {
        my ( $module_number, $routine ) =
          split ( ":", $Modules_Install[$x] );
        print STDERR "$x - $module_number - $routine\n";
    }

    print STDERR "\nModules Post Install ---------------\n";
    for ( my $x = 0 ; $x <= $#Modules_Post_Install ; $x++ ) {
        my ( $module_number, $routine ) =
          split ( ":", $Modules_Post_Install[$x] );
        print STDERR "$x - $module_number - $routine\n";
    }
    print STDERR "\nModules Post Validate ---------------\n";
    for ( my $x = 0 ; $x <= $#Modules_Post_Validate ; $x++ ) {
        my ( $module_number, $routine ) =
          split ( ":", $Modules_Post_Validate[$x] );
        print STDERR "$x - $module_number - $routine\n";
    }

}

sub parse_xml_files ( $$ ) {
###################################################################
    #                                                                 #
###################################################################
    my ( $core_spec, $vendor_spec ) = @_;

    #
    if ($test_debug) { print STDERR ">> Entering parse_xml_files \n"; }

    #
    if ( -f $core_spec ) {
        $Spec_Type = "CORE";
        $spec_twig->parsefile($core_spec);
        my $root = $spec_twig->root;
    }

    $spec_twig->purge;

    if ( -f $vendor_spec ) {
        $Spec_Type = "VENDOR";
        $spec_twig->parsefile($vendor_spec);
        my $root = $spec_twig->root;
    }

    $spec_twig->purge;

    if ($test_debug) { &dump_modules_array; }
}

sub return_core_defaults ( ) {
###################################################################
    #                                                                 #
###################################################################
    #
    if ($test_debug) { print STDERR ">> Entering return_core_defaults\n"; }

    #
    return (
        $Core_Default_DBCONNECT,       $Core_Default_BASEDIR,
        $Core_Default_CORE_INSTALLDIR, $Core_Default_PORTBASE,
        $Core_Default_LICENSEHOST,     $Core_Default_TMPDIR,
        $Core_Default_CLASSPATH
    );
}

sub return_module_overrides ( ) {
###################################################################
    #                                                                 #
###################################################################
    #
    if ($test_debug) { print STDERR ">> Entering return_module_overrides\n"; }

    #
    return (
        $Module_Default_DBCONNECT,       $Module_Default_BASEDIR,
        $Module_Default_CORE_INSTALLDIR, $Module_Default_PORTBASE,
        $Module_Default_LICENSEHOST,     $Module_Default_TMPDIR,
        $Module_Default_CLASSPATH
    );
}

sub update_globals ( $ ) {
###################################################################
    #                                                                 #
###################################################################

    my ($old_installdir) = @_;

    #
    if ($test_debug) { print STDERR ">> Entering update_globals\n"; }

    #
    for ( my $x = 0 ; $x <= $#Global_Pre_Validate ; $x++ ) {
        $_ = $Global_Pre_Validate[$x];
        s/^$old_installdir/$Core_Default_CORE_INSTALLDIR/g;
        $Global_Pre_Validate[$x] = $_;
    }

    for ( my $x = 0 ; $x <= $#Global_Pre_Install ; $x++ ) {
        $_ = $Global_Pre_Install[$x];
        s/^$old_installdir/$Core_Default_CORE_INSTALLDIR/g;
        $Global_Pre_Install[$x] = $_;
    }

    for ( my $x = 0 ; $x <= $#Global_Post_Install ; $x++ ) {
        $_ = $Global_Post_Install[$x];
        s/^$old_installdir/$Core_Default_CORE_INSTALLDIR/g;
        $Global_Post_Install[$x] = $_;
    }

    for ( my $x = 0 ; $x <= $#Global_Post_Validate ; $x++ ) {
        $_ = $Global_Post_Validate[$x];
        s/^$old_installdir/$Core_Default_CORE_INSTALLDIR/g;
        $Global_Post_Validate[$x] = $_;
    }

}

sub update_defaults ( $$$$$$$ ) {
###################################################################
    #                                                                 #
###################################################################
    my (
        $ret_DBCONNECT, $ret_BASEDIR,     $ret_CORE_INSTALLDIR,
        $ret_PORTBASE,  $ret_LICENSEHOST, $ret_TMPDIR, $ret_CLASSPATH
      )
      = @_;

    #
    if ($test_debug) { print STDERR ">> Entering update_defaults\n"; }

    #

    $Core_Default_DBCONNECT = $ret_DBCONNECT if ($ret_DBCONNECT);
    $Core_Default_BASEDIR   = $ret_BASEDIR   if ($ret_BASEDIR);

    if ($ret_CORE_INSTALLDIR) {
        my ($old_installdir) = $Core_Default_CORE_INSTALLDIR;
        $Core_Default_CORE_INSTALLDIR = $ret_CORE_INSTALLDIR;
        &update_globals($old_installdir);
    }

    $Core_Default_PORTBASE    = $ret_PORTBASE    if ($ret_PORTBASE);
    $Core_Default_LICENSEHOST = $ret_LICENSEHOST if ($ret_LICENSEHOST);
    $Core_Default_TMPDIR      = $ret_TMPDIR      if ($ret_TMPDIR);
    $Core_Default_CLASSPATH   = $ret_CLASSPATH   if ($ret_CLASSPATH);

    $ENV{'PORT_GROUP'}  = $Core_Default_PORTBASE;
    $ENV{'WM_PM_PORT'}  = $Core_Default_PORTBASE + 1;
    $ENV{'DB_CONNECT'}  = $Core_Default_DBCONNECT;
    $ENV{'SETUP'}       = "fixed";
    $ENV{'FLEXPM_HOME'} = join "/", $ret_BASEDIR, $ret_CORE_INSTALLDIR;
    my (@db_string) = split('/',$Core_Default_DBCONNECT);
    my (@db_password) = split('@',$db_string[1]);
    $ENV{'DBUSER'}     = $db_string[0];
    $ENV{'DBPASSWD'}   = $db_password[0];
    $ENV{'DBSID'}      = $db_password[1];


}

sub return_tarballs ( ) {
###################################################################
    #                                                                 #
###################################################################
    my (@tar_array);

    #
    if ($test_debug) { print STDERR ">> Entering return_tarballs\n"; }

    #
    for ( my $x = 0 ; $x <= $#Modules_Module ; $x++ ) {
        my $value    = $Modules_Module[$x]->{TARFILE};
        my $name     = $Modules_Module[$x]->{NAME};
        my $process  = $Modules_Module[$x]->{PROCESS_TYPE};
        my $inst_dir = "";
        if ( $name eq "CORE" ) {
            $inst_dir = $Core_Default_CORE_INSTALLDIR;
        }
        else {
            $inst_dir = $Modules_Module[$x]->{MODULE_INSTALLDIR};
        }

        if ( $value ne "" ) {
            push @tar_array, join ( ":", $name, $inst_dir, $value, $process );
        }
        else {
            push @tar_array, join ( ":", $name, $inst_dir, "NULL", $process );
        }
    }
    return (@tar_array);
}

sub update_tarballs ( @ ) {
###################################################################
    #                                                                 #
###################################################################
    my (@tar_array) = @_;

    #
    if ($test_debug) { print STDERR ">> Entering update_tarballs\n"; }

    #

    for ( my $x = 0 ; $x <= $#tar_array ; $x++ ) {
        my ( $name, $inst_dir, $value, $process ) =
          split ( ":", $tar_array[$x] );
        $Modules_Module[$x]->{PROCESS_TYPE} = $process;
    }
}

sub return_global_pre_validate ( )      { return (@Global_Pre_Validate);       }

sub return_global_pre_install ( )       { return (@Global_Pre_Install);        }

sub return_global_post_install ( )      { return (@Global_Post_Install);       }

sub return_global_post_validate ( )     { return (@Global_Post_Validate);      }

sub return_number_of_modules ( )        { return ($#Modules_Module);           }
sub return_original_core_installdir( )  { return ($Original_Core_INSTALLDIR);  }

sub return_modules_module_prevalidate ( $ ) {
###################################################################
    #                                                                 #
###################################################################
    my ($mod_number) = @_;
    my (@routines);

    for ( my $x = 0 ; $x <= $#Modules_Pre_Validate ; $x++ ) {
        my ( $module_number, $script, $lang ) =
          split ( ":", $Modules_Pre_Validate[$x] );
        if ( $module_number eq $mod_number ) {
            push @routines, join(":",$script,$lang);
        }
    }
    if ($test_debug) {
        print STDERR " >> return_modules_module_prevalidate \n";
        print STDERR " >>> Num of Modules_Pre_validate ($#Modules_Pre_Validate) @routines\n";
    }

    return (@routines);
}

sub return_modules_module_preinstall ( $ ) {
###################################################################
    #                                                                 #
###################################################################
    my ($mod_number) = @_;
    my (@routines);

    for ( my $x = 0 ; $x <= $#Modules_Pre_Install ; $x++ ) {
        my ( $module_number, $script,$lang ) =
          split ( ":", $Modules_Pre_Install[$x] );
        if ( $module_number eq $mod_number ) {
            push @routines, join(":",$script,$lang);
        }
    }
    if ($test_debug) { print STDERR " >> return_modules_module_preinstall \n"; 
                       print STDERR " >>> Num of Modules_Pre_install ($#Modules_Pre_Install) @routines\n"; }
    return (@routines);
}

sub return_modules_module_install ( $ ) {
###################################################################
    #                                                                 #
###################################################################
    my ($mod_number) = @_;
    my (@routines);

    for ( my $x = 0 ; $x <= $#Modules_Install ; $x++ ) {
        my ( $module_number, $script,$lang ) = split ( ":", $Modules_Install[$x] );
        if ( $module_number eq $mod_number ) {
            push @routines, join(":",$script,$lang);
        }
    }
    if ($test_debug) { print STDERR " >> return_modules_module_install \n"; }
    return (@routines);
}

sub return_modules_module_postinstall ( $ ) {
###################################################################
    #                                                                 #
###################################################################
    my ($mod_number) = @_;
    my (@routines);

    for ( my $x = 0 ; $x <= $#Modules_Post_Install ; $x++ ) {
        my ( $module_number, $script,$lang ) =
          split ( ":", $Modules_Post_Install[$x] );
        if ( $module_number eq $mod_number ) {
            push @routines, join(":",$script,$lang);
        }
    }
    if ($test_debug) {
        print STDERR " >> return_modules_module_post_install \n";
    }
    return (@routines);
}

sub return_modules_module_postvalidate ( $ ) {
###################################################################
    #                                                                 #
###################################################################
    my ($mod_number) = @_;
    my (@routines);

    for ( my $x = 0 ; $x <= $#Modules_Post_Validate ; $x++ ) {
        my ( $module_number, $script,$lang ) =
          split ( ":", $Modules_Post_Validate[$x] );
        if ( $module_number eq $mod_number ) {
            push @routines, join(":",$script,$lang);
        }
    }
    if ($test_debug) {
        print STDERR " >> return_modules_module_post_validate \n";
    }
    return (@routines);
}

sub return_vendor_modules_module ( ) {
###################################################################
    #                                                                 #
###################################################################
    my (@Install_Dirs);
    my ($value);

    for ( my $x = 0 ; $x <= $#Modules_Module ; $x++ ) {
        $value = $Modules_Module[$x]->{NAME};
        next if ( $value eq "CORE" );
        $value = $Modules_Module[$x]->{MODULE_INSTALLDIR};
        push @Install_Dirs, $value unless $value eq "";
    }

    return (@Install_Dirs);

}

sub return_core_modules_module ( ) {
###################################################################
#                                                                 #
###################################################################
    return (
        $Core_Module_NAME,    $Core_Module_PLATFORM, $Core_Module_VERSION,
        $Core_Module_TARFILE, $Core_Module_SPACE
    );
}

sub return_prerequisite_versions ( $ ) {
###################################################################
#                                                                 #
###################################################################
    my ($tarball_number) = @_;

    return ($Modules_Module[$tarball_number]->{SPEC_TYPE},
            $Modules_Module[$tarball_number]->{NAME}, 
            $Modules_Module[$tarball_number]->{PLATFORM},
            $Modules_Module[$tarball_number]->{VERSION},
            $Modules_Module[$tarball_number]->{PROCESS_TYPE},
            $Modules_Module[$tarball_number]->{PREREQ_CORE_PLATFORM},
            $Modules_Module[$tarball_number]->{PREREQ_CORE_VERSION},
            $Modules_Module[$tarball_number]->{PREREQ_MODULE_PLATFORM},
            $Modules_Module[$tarball_number]->{PREREQ_MODULE_VERSION}
           );

}

1;
