#!/bin/ksh
#> This script will uninstall Siemens UMTS patch 
#> patch: 5.0.8.0.10 
#

# variables used after <productVersion> and <wsVersion> drop.
export NEPATCH_START_DIR=`pwd`
export ARCHIVE_DIR=$NEPATCH_START_DIR/Archive
export BACKUP_DIR=$NEPATCH_START_DIR/Backup

########################################################
# Check system values required for this module but not checked by the
# main installer.
if [[ -z $SieGSM_HOME ]]; then
  echo "+> SieGSM_HOME environment is not defined"
  echo "+> Please ensure that this is defined and then reinstall upgrade"
  echo ""
  exit 2
fi

if [[ ! -d $SieGSM_HOME ]]; then
  echo "+> SieGSM_HOME environment value '$SieGSM_HOME'"
  echo "+> is not a valid directory. Please check this value"
  echo ""
  exit 2
fi


patch_num="4.0.4.0.10"
cd `dirname $0`
patch_dir=$PWD

date +"+ Starting uninstall of patch $patch_num at %T, %D ..."
echo "@$patch_dir/uninstall.sql" | sqlplus $DB_CONNECT

retcode=$?

if [[ $retcode != 0 ]]; then
    echo "+++ Could not revert component history. Perhaps patch $patch_num is not installed, or another patch has been installed since."
    echo "+ Exiting..."
    exit 2
fi

backup_dir=$FLEXPM_HOME/patches/$patch_num/SieGSM/Backup
archive=`ls -t $backup_dir/Module_Archive_*.tar.gz|head -1`

if [[ ! -f "$archive" ]]; then
    echo "+++ Backup for patch $patch_num not found!"
    echo "+ Exiting..."
    exit 2
fi

echo "+++ Restoring changed files from backup..."
gunzip -c $archive | (cd $SieGSM_HOME; tar xvf -)


echo "+ Finished uninstalling patch ..."

date +"+ Uninstall of $patch_num is completed at %T, %D ....."
echo ""
exit 0
