-- Amend the component installation history to remove
-- patch 4.0.4.0.10
-- After this program finishes, the patch will no longer appear in
-- the output of show_installed or show_history.
-- Note this program will work only if nothing else has been installed
-- for the SieGSM module since 4.0.4.0.10

WHENEVER SQLERROR EXIT FAILURE ROLLBACK
set serveroutput on
declare
   ci       component.component_id%TYPE;
   mcvi     component_version.component_version_id%TYPE;
   dummy    component_version.component_version_id%TYPE;
   prev_cvi component_version.component_version_id%TYPE;
begin

   select component_id into ci from component where component_name = 'SieGSM';
   -- Find the latest thing installed for SieGSM
   select max(component_version_id) into mcvi from component_version where component_id = ci;

   -- Make sure 4.0.4.0.10 was the last thing installed -
   -- this will throw NO_DATA_FOUND if it's something else.
   select 1 into dummy from component_version where component_version_id = mcvi and revision = '4.0.4.0.10';

   -- Make sure 4.0.4.0.10 hasn't already been uninstalled -
   -- this too will throw NO_DATA_FOUND if it has been.
   select 1 into dummy from component_installed where component_version_id = mcvi;

   -- Find the previous thing installed for SieGSM
   select max(component_version_id) into prev_cvi from component_version where component_id = ci and component_version_id != mcvi;

   -- Remove the "start" and "finish" records from the output of
   -- show_history.
   delete from component_history where component_version_id = mcvi;

   -- Make the previous SieGSM install show up in the output of
   -- show_installed.
   update component_installed set component_version_id = prev_cvi where component_version_id = mcvi;

   -- Remove the component_version 4.0.4.0.10
   delete from component_version where component_version_id = mcvi and revision = '4.0.4.0.10';

   commit;
end;
/
