#!/usr/bin/env perl -w

#########################################################################
# @(#) filedecode.pl 1.16 0 07/10/30 17:47:43
# @(#) 
#########################################################################
## This program takes the following parameters :
## 1) -f <file name> 
##       Will return the following information in XML format:
##       instance_id, entity_id, file_type_id, start_tstamp, numofinterval.
## 2) -e <bsc name> [-t <filetype>] 
##       Will return instance_id value(s) for a given filetype.
##       If a filetype is not provided ( via the -t option ), or if the
##       filetype provided is invalid, all instance ids that are associated
##       with the given ne name will be printed in a comma-separated line
#########################################################################

use lib $ENV{FLEXPM_HOME}."/sw/perl/lib"; # location of the DBI modules

use strict;
use vars qw($opt_f $opt_e $opt_t);
use Getopt::Std;
use File::Basename;
use Env ;
use DBI;

# most of variables declared below are global ones used in different procedures
my $ElementList = "${PROSPECT_HOME}/element_list";
my (%fileTypeHash);
my ($instance_id, $entity_id, $file_type_id, $new_datetime, $time_stamp);
my ($sysid, $mscid, $bscid, $rncid);
my ($filename, $nename, $shortFile);
my ($prefix, $path, $filetype);
my $keyDelimiter = "-";
my $innerKeyDelimiter = "~";

# These are used for configuration information used in preprocessing and decoding data.
my $configHash = {} ;
my $bscID = "";
my $R55DBMapping="$SieGSM_HOME/vendor/Siemens/GSM/1.3/SieGSM_loader/SIEMENSBSCASCII/R55/data/dbMapping.csv" ;
my $R55fmMapping="$SieGSM_HOME/vendor/Siemens/GSM/1.3/SieGSM_loader/SIEMENSBSCASCII/R55/data/fmMapping.csv" ;
my $R60Config="$SieGSM_HOME/vendor/Siemens/GSM/1.3/SieGSM_loader/SIEMENSBSCASCII/data/preLoadConfig.csv" ;
readConfiguration($R60Config, $configHash) ;

# Set the base entiy to be used
my $baseEntity='BSC';
my $SieCSBaseEntity='SAM';    #SieCS

# Split the command line arguments
if (!getopts('f:e:t:'))
{
   usage("Invalid number of arguments");
   exit 2;
}

# Set the filename formats use
$fileTypeHash{"SIEMENSBSCASCII"}=".ASCII";
$fileTypeHash{"SIEMENSBSCASCII_R55"}=".exp";
$fileTypeHash{"SIEMENSBSCXML"}=".xml";
$fileTypeHash{"SPATIALXML"}=$SieCSBaseEntity.".xml";   #SieCS
$fileTypeHash{"SPATIALATRIUMASCII"}=$SieCSBaseEntity.".ASCII";   #SieCS
$fileTypeHash{"RNC_PM"}=".RNCpm";
$fileTypeHash{"RNC_CFG"}=".CFGpm";
$fileTypeHash{"NodeB_PM"}=".NODEBpm";

# Check the element_list is in place
&CheckList();

# Connect to oracle
my $dbh = connect_to_oracle();

# Process the supplied arguments
#---------------------
if (defined $opt_f)
#---------------------
{
   $filename=$opt_f;
   
   # Check that the datafile is not empty and exists
   &CheckFile($filename);

   # Split the filename into its separate parts
   &SplitFileName($filename);   

   # Get the converted tstamp from the filename
   &GetTstamp();

   # Get the filetype from the filename and filetype hash
   &GetFileTypeID();

   # get System, MSC and BSC ID for searching other info
   &GetSystemMscBscID();  

   # Get the Instance entity id from the database.
   &GetInstanceEntityID();

   # Print the XML output
   &printXML();
}
elsif (defined $opt_e)
{
   # This is likely to return an number of instance_id as we are using BSC, and
   # they are not unique. (MSC-BSC MSC is logical only-no data)
   $nename=$opt_e;
   my $full_key;

   if ( defined $opt_t ) {
      &GetFileTypeInfo ( $opt_t );
   }

   ($full_key, $baseEntity) = get_fullkey_ent_base ($nename);

   if ($baseEntity eq $SieCSBaseEntity)
   {
      print STDOUT &GetSAMInstanceID($nename, $full_key, $baseEntity)."\n";
   }
   else
   {
      print STDOUT &GetBSCInstanceID()."\n";
   }

}
else
{
   $dbh->disconnect;
   usage("Invalid option");
}

$dbh->disconnect;
exit(0);

##############################################################
sub get_fullkey_ent_base
##############################################################
{
   my ($ne_name) = @_;
    
   my $line;
   my $base_entity=$baseEntity;
   my $full_key;

   open IN, "<$ElementList" or terminalError($ne_name, "Couldn't open element_list");
   while (not defined $full_key and $line=<IN>) {
      chomp $line ;
      next if $line =~ /^\#/ or $line =~ /^\s*$/; # Skip comments and blank lines.

      my ($element_name, $sys_id, $element_id, $release, $buffer_size, $entity_type, $tech_type) = split ('\t', $line);

     if ($ne_name eq $element_name) {
        if ($element_id ne '' and $sys_id ne '') {
           $element_name =~ s/$keyDelimiter/$innerKeyDelimiter/g;	   
           $full_key = $sys_id . '-'. $element_name;
	   $full_key = uc ($full_key);
	   $base_entity = $entity_type;

           }
       }
   }
   close(IN);

   if(not defined $full_key){
     $full_key="";
   }

   return ($full_key, $base_entity);

}


# Retrieve the System/MSC/BSC
#------------------------------------------------------------------------------
sub GetSystemMscBscID()
#------------------------------------------------------------------------------
{
   my (@mscnamearr, @sys_msc);
   my ($i, $mscname, $allmscs, $OMPID,$return_string);
   my @return_val;
   my $fname;

   if ( $filename =~ /.tar.gz$/ ) {
      $fname = substr($filename, 0, length($filename)-7);
   } elsif ($filename =~ /.tar$/ ) {
     $fname = substr($filename, 0, length($filename) - 4);
   } elsif ($filename =~ /.gz$/ )
   {
      $fname = substr($filename, 0, length($filename)-3);
   } else {
      $fname = $filename;
   }

   # Need to do different things for different versions of data and with different
   # loading options used.
   if ($filename =~ /.exp$/ ) # R5.5 data
   {
        undef $allmscs;
        $allmscs = `grep "\^$nename" $ElementList | awk '{print \$1, \$2, \$3 }'`;
        @mscnamearr = split(/\n/, $allmscs);
        for ($i = 0; $i <= $#mscnamearr; $i++)
        {
           @sys_msc = split(/\s+/, $mscnamearr[$i]);
    
           if ($nename eq $sys_msc[0])
           {
              $sysid = $sys_msc[1];
              last;
           }
        }
   } elsif ( $filename =~ /.RNCpm/ || $filename =~ /.CFGpm/  || $filename =~ /.NODEBpm/ ) {
     open IN, "<$ElementList" or die "Couldn't open element_list";
     while ( my $line = <IN> ) {
       chomp($line);
       if ( $line =~ /^${nename}\t/ ) {
         my @fields=split('\t',$line);
         if ( $fields[0] eq $nename && $fields[5] eq 'RNC') {
           $sysid = $fields[1];
           return;
         }
       }
     }
     close IN;
     
   } elsif ( $baseEntity eq $SieCSBaseEntity ) {
     open IN, "<$ElementList" or die "Couldn't open element_list";
     while ( my $line = <IN> ) {
       chomp($line);
       if ( $line =~ /^${nename}\t/ ) {
         my @fields=split('\t',$line);
         if ( $fields[0] eq $nename) {
           $sysid = $fields[1];
           return;
         }
       }
     }
     close IN;
   } else { # R6.0
      # retrieve System and MSC IDs from msc_list
   
      undef $allmscs;
      $allmscs = `grep "\^$nename" $ElementList | awk '{print \$1, \$2, \$3 }'`;
 # print "$allmscs\n"; 
      if ($allmscs eq "")
      {
         if ( exists($configHash->{MSCPREFIX}) )
         {
            $mscname=$configHash->{MSCPREFIX}.$mscid;
	    print STDOUT "$mscname";
         }
         $allmscs = `grep "\^$mscname" \$ElementList | awk '{print \$1, \$2, \$3 }'`;
         if ($allmscs eq "")
         {
            terminalError("No match found for $mscname in element_list"); 
         }
      }
   
      @mscnamearr = split("\n", $allmscs);
      #print STDOUT "$mscnamearr[0]\n"; 
      undef $sysid;
      for ($i = 0; $i <= $#mscnamearr; $i++)
      {
         @sys_msc = split(/\s+/, $mscnamearr[$i]);
   
         if ($nename eq $sys_msc[0])
         {
	# print "$nename\n";
            $sysid = $sys_msc[1];
            # Remove leading 0's if possible from BSC/MSC ID's
   
            if ( $mscid =~ /^\d+$/ )
            {
               if ( exists($configHash->{MSCPREFIX}) )
               {
                  $mscid=$configHash->{MSCPREFIX}.int($mscid);
               } else {
                  $mscid = int($mscid);
               }
            }
            
            if ( exists($configHash->{USEEXTERNALBSC}) )
            {
               if ( $bscid =~ /^\d+$/ )
               {
                  $bscid = int($bscid);
               }            
            } else {
               $bscID = decode_R60_data();
            }
            if ($fname =~ /.xml$/ ) #R8.0
	    {
	    $bscID = decode_R80_data();
	    }
            $bscid = bscid_to_load($bscid);
            
            last;
         }
      }
   }

   chomp($sysid);
   chomp($mscid);
   chomp($bscid);

   if ((not defined $sysid) || (not defined $mscid) || (not defined $bscid))
   {
      terminalError("Can not find exact msc name $mscname in element_list"); 
   }
}

#------------------------------------------------------------------------------
sub GetInstanceEntityID()
#------------------------------------------------------------------------------
{
   my ($inst_ent_info, $inst_ent_id);
   my ($fullkey);

   if ( $filename =~ /.RNCpm/ || $filename =~ /.CFGpm/ || $filename =~ /.NODEBpm/ ) {
     $fullkey = join( '-' , $sysid, $rncid );
     $baseEntity = 'RNC';
   }elsif ($baseEntity eq $SieCSBaseEntity){
     $nename =~ s/$keyDelimiter/$innerKeyDelimiter/g;
     $fullkey = join( '-' , $sysid, uc($nename) );
   }
   else {
     $fullkey = join( '-', $sysid, uc($mscid), $bscid );
   } 
   #print "entity_id = $entity_id, baseEntity=$baseEntity,SieCSBaseEntity=$SieCSBaseEntity \n";
   #print "fullkey = $fullkey and nename = $nename \n";
   my $query = $dbh->prepare("select instance_id,entity_id from wmn_instance_context where fullkey =? and entity_id = wm_Network.Get_Entity(?) and context_id = (select context_id from wmn_context where context_name like '\%Generic\%')");

   unless (defined $query) {
     terminalError("ORACLE : Cannot prepare select instance id query : $DBI::errstr");
   }

   unless ( $query->execute($fullkey,$baseEntity) ) {
     terminalError("ORACLE : Failed to execute select instance id query : $DBI::errstr");
   }

   $query->bind_columns(\$instance_id, \$entity_id);

   if ( ! $query->fetch) {
     terminalError("Problem getting instance_id or entity_id from wmn_instance_context table");
   }
}


# Get the BSC id from the database, this could return a number of ID's
#------------------------------------------------------------------------------
sub GetBSCInstanceID()
#------------------------------------------------------------------------------
{
  my ($inst_ent_info, $inst_ent_id);
  my (@instances, $instance);
  # if condition introduced to handle instance name request from add_filetype_timeout.sh -i option.

  my $query = $dbh->prepare("select instance_id from wmn_instance where instance_name = ? and entity_id = wm_Network.Get_Entity( ? )");

  unless (defined $query) {
    terminalError("ORACLE : Cannot prepare select all instance_id query : $DBI::errstr");
  }

  unless ( $query->execute( $nename, $baseEntity ) ) {
    terminalError("ORACLE : Failed to execute select all instance_id query : $DBI::errstr");
  }

  $query->bind_columns(\$instance);

  while ( $query->fetch )
  { 
    push @instances,  $instance;
  }

  my ( @instances1 );
  $query = $dbh->prepare("select instance_id from wmn_instance_context where localkey = ? and entity_id = wm_Network.Get_Entity( ? ) and context_id = ( select context_id from wmn_context where context_name like '\%Generic\%')");

  unless (defined $query) {
    terminalError("ORACLE : Cannot prepare select all instance_id query : $DBI::errstr");
  }

  unless ( $query->execute( $nename, $baseEntity ) ) {
    terminalError("ORACLE : Failed to execute select all instance_id query : $DBI::errstr");
  }

  $query->bind_columns(\$instance);

  while ( $query->fetch )
  { 
    push @instances1,  $instance;
  }

  if (defined $instances1[2]) {
    if ($instances1[2] =~ /[0-9]/ ){
      @instances = @instances1;
    }
  } elsif ( defined $instances1[0] && $instances1[0] =~ /[0-9]/) {
    @instances = @instances1;
  }

  # This should capture any requests for RNC data 
  if ( ! @instances ) {
    $query = $dbh->prepare("select instance_id from wmn_instance where label = ? and entity_id = wm_Network.Get_Entity( ? )");

    unless (defined $query) {
      terminalError("ORACLE : Cannot prepare select all instance_id query : $DBI::errstr");
    }

    unless ( $query->execute( $nename, 'RNC' ) ) {
      terminalError("ORACLE : Failed to execute select all instance_id query : $DBI::errstr");
    }

    $query->bind_columns(\$instance);

    while ( $query->fetch )
    { 
      push @instances,  $instance;
    }
  }
  $inst_ent_info = "";
   
  foreach $inst_ent_id (@instances)
  {
    chomp($inst_ent_id);
    if ($inst_ent_id =~ /ORA-\d+/ || $inst_ent_id =~ /SP2/)  
    {
       terminalError("$inst_ent_info");
    }
    elsif ($inst_ent_id =~ /\d+/)
    {
      if ( $inst_ent_info eq "")
      {
         $inst_ent_info = int($inst_ent_id);
      } else {
         $inst_ent_info = $inst_ent_info.", ".int($inst_ent_id);
      }
    } elsif ($inst_ent_id =~ /Connected./ || $inst_ent_id eq "")  {
       # do nothing
    }
    else
    {
       terminalError("Problem getting instance_id from wmn_instance_context table");
    }
  }
   
  if ( $inst_ent_info eq "" )
  {
    terminalError("Problem getting instance_id from wmn_instance_context table");
  } else {
    return $inst_ent_info;
  }
}


# Get the SAM id from the database, this could return a number of ID's
#------------------------------------------------------------------------------
sub GetSAMInstanceID()
#------------------------------------------------------------------------------
{
   my ($ne_name, $full_key, $base_entity) = @_;
   my $query;
   if ($base_entity eq $SieCSBaseEntity) 
   {
      $query = $dbh->prepare("select instance_id from wmn_instance_context where fullkey = ? and entity_id = wm_Network.Get_Entity(?) and context_id = (select context_id from wmn_context where context_name like '%Generic%')");
      unless (defined $query) {
         terminalError("ORACLE","Cannot prepare select all instance_id query : $DBI::errstr");
      }
      unless ( $query->execute( $full_key, $base_entity) ) {
         terminalError("ORACLE","Failed to execute select all instance_id query : $DBI::errstr");
      }
   }

  my @instances;
  my $instance;

  $query->bind_columns(\$instance);

  while ( $query->fetch )
  {
      push @instances,  $instance;
  }

  unless (scalar @instances == 0) {
  return join " ", @instances;
  } 
  else {
  terminalError($ne_name, "00Problem getting instance_id from wmn_instance_context table");
  }
}



# Get the file type id from the database using the hash defined earlier.
#------------------------------------------------------------------------------
sub GetFileTypeID()
#------------------------------------------------------------------------------
{
   my ($fileTypeName, $fileTypeValue, $fileTypeInfo);
   undef $file_type_id;
   foreach $fileTypeName (keys(%fileTypeHash))
   {
      $fileTypeValue = $fileTypeHash{$fileTypeName};

      #if ($filetype =~ m/$fileTypeValue/)
      if ($filetype eq $fileTypeValue)  #added to check SieCS
      {
         if ( $fileTypeName =~ /^SIEMENSBSC/ ) {
           $fileTypeName='SIEMENSBSC';
         } 

         my $query = $dbh->prepare("select file_type_id from wm_file_control where file_type=?"); 

         unless (defined $query) {
           terminalError("ORACLE : Cannot prepare select file_type_id query : $DBI::errstr");
         }

         unless ( $query->execute($fileTypeName) ) {
           terminalError("ORACLE : Failed to execute select file_type_id query : $DBI::errstr");
         }

         $query->bind_columns(\$file_type_id);

         if (! $query->fetch) {
           terminalError($filename, "Problem getting file_type_id from wm_file_control table for file type $fileTypeName");
         }
      }
   }

   if (not defined $file_type_id)
   {
      terminalError("Invalid file type in $filename");
   }
}

# Convert the date.
#------------------------------------------------------------------------------
sub ConvertDate
#------------------------------------------------------------------------------
{
  my $filedate = $_[0];
  my $datetime_info;
  my $query = $dbh->prepare("select to_char(to_date('$filedate', 'yyyymmddhh24miss'), 'yyyy/mm/dd hh24:mi:ss') from dual");

  unless (defined $query) {
    terminalError("ORACLE : Cannot prepare select file_type_id query : $DBI::errstr");
  }

  unless ( $query->execute() ) {
    terminalError("Problem with timestamp field $filedate provided in $filename");
  }
  
  $query->bind_columns( \$datetime_info ); 

  if ($query->fetch) {
    $new_datetime =  $datetime_info;
  } else {
    terminalError("Problem with timestamp field $filedate provided in $filename");
  }
}

# Print out the XML message
#------------------------------------------------------------------------------
sub printXML()
#------------------------------------------------------------------------------
{
  # Print out XML
  print STDOUT "<FILEDECODE Filename=\"$filename\">\n";
  print STDOUT "  <ATTRIBUTE Name=\"instance_id\" Value=\"$instance_id\"\/>\n";
  print STDOUT "  <ATTRIBUTE Name=\"entity_id\" Value=\"$entity_id\"\/>\n";
  print STDOUT "  <ATTRIBUTE Name=\"file_type_id\" Value=\"$file_type_id\"\/>\n";
  print STDOUT "  <ATTRIBUTE Name=\"start_tstamp\" Value=\"$new_datetime\"\/>\n";
  print STDOUT "  <ATTRIBUTE Name=\"numofinterval\" Value=\"1\"\/>\n";
  print STDOUT "</FILEDECODE>\n";
}

# Show usage
#------------------------------------------------------------------------------
sub usage {
#------------------------------------------------------------------------------
  my $message = $_[0];
  my $progName = `basename $0`;
  chomp $progName;

  print STDERR "\nError: $message\n\n";
  print STDERR "Usage: $progName { -f <file name> | -e <ne name> }\n\n";
  print STDERR "   -f <file name>: will return the following information in XML format -\n";
  print STDERR "      instance_id, entity_id, file_type_id, start_tstamp, and numofinterval.\n\n";
  print STDERR "   -e <ne name> [-t <filetype>]:  will return instance_id value.\n\n";
  exit(1);
}

#------------------------------------------------------------------------------
sub terminalError {
#------------------------------------------------------------------------------
  my $message = $_[0];

  print STDERR "\nError. $message.\n\n";
  exit(1);
}

## Function: readConfiguration - reads the configuration file and builds
# a hash based on the information needed.
#------------------------------------------------------------------------------
sub readConfiguration {
#------------------------------------------------------------------------------
  my ($confFile, $configHash) = @_ ;
  my ($line) ;

  if (!open(CONFIG, $confFile)) {
    usage("Could not open configuration file: $confFile, for reading") ;
  }

  # Process every line in configuration file, ignoring comments and
  # blanks.
  while ($line = <CONFIG>) {
    chomp $line ;
    next if ($line eq '') ; # Skip line if it is a comment or blank
    next if ($line =~ /^\#/) ;

    # Split configuration line into token and value
    my ($token, $value) ;
    if (!($line =~ /^(\w+),(.*)$/ && ($token = $1, $value = $2))) {
      print "\nIgnoring invalid line in configuration file at line:\n" ;
      print "\n".$line."\n" ;
      next ;
    }

    # If it is a comment the value may well be over a number of lines,
    # force this to be the case.
    if ($token =~ /COMMENT/) {
      $value =~ s/\\n/\n/g ;
    }

    # If it is an SCCS comment the value will contain \% which need
    # to be substituted with %.
    if ($token =~ /SCCS/) {
      $value =~ s/\\%/%/g ;
    }

    # If it is a filepath the value may well contain $PROSPECT_HOME,
    # in this case expand if present.
    if ($token =~ /MAP/) {
      $value =~ s/\$PROSPECT_HOME/$PROSPECT_HOME/g ;
    }

    # Build configuration hash using token as the index
    $configHash->{$token} = $value ;
  }
  close(CONFIG) ; # Close configuration file and exit
}

# Checks which BSC id should be loaded
#------------------------------------------------------------------------------
sub bscid_to_load {
#------------------------------------------------------------------------------
  my $databsc = $_[0] ;
  my $bscid ;

  if ( exists($configHash->{USEEXTERNALBSC}) )
  {
    if ( exists($configHash->{BSCPREFIX}) )
    {
      $bscid="$configHash->{BSCPREFIX}".$databsc;
    } else {
      $bscid=$databsc;
    }
  } else {
    $bscid=$bscID;
  }

  return $bscid ;
}

# Decode the R55 data and return the MSC/BSC ids
#------------------------------------------------------------------------------
sub decode_R55_data {
#------------------------------------------------------------------------------
   my ( $tmp_MSCname, $bsc );
   my @return_val;
   my @allbscs;
   my $return_value;

   CheckR55Files();
   
   @return_val = split (/\_/,$shortFile);
   $tmp_MSCname=$return_val[1];
   
   # Need to do a lookup in the files now...
   @allbscs=`grep "$tmp_MSCname,Cell" $R55DBMapping | awk -F, '{ print \$6 }' | awk -F- '{ print \$3","\$4 }'`;
   
   foreach $bsc (@allbscs)
   {
      # for all of these we need to grep the BSC/MSC from the fmMapping.csv
      chomp $bsc;
      $return_value=`grep $bsc $R55fmMapping | awk -F, '{ print \$6"-"\$5 }'`;
      if ( $return_value ne "" )
      {
         last;
      }
   }

   return $return_value;
}

# Decode the R60 data and return the MSC/BSC ids
#------------------------------------------------------------------------------
sub decode_R60_data {
#------------------------------------------------------------------------------
   my ( $tmp_MSCname, $bsc );
   my @return_val;
   my @allbscs;
   my $return_value;
   my $fname;
   if ($filename =~ /.gz$/ ){
    `mkdir $PROSPECT_HOME/../ftpIN/SIEMENSBSCASCII/in/temp_$$`;
    `cp $filename $PROSPECT_HOME/../ftpIN/SIEMENSBSCASCII/in/temp_$$`;
    `gunzip $PROSPECT_HOME/../ftpIN/SIEMENSBSCASCII/in/temp_$$/*.gz`;
    $fname = substr($filename, 0, length($filename)-3);
    @allbscs=`grep "bsc {" $PROSPECT_HOME/../ftpIN/SIEMENSBSCASCII/in/temp_$$/*`;
    `rm -rf $PROSPECT_HOME/../ftpIN/SIEMENSBSCASCII/in/temp_$$`;
   } else {
    $fname = $filename;
    @allbscs=`grep "bsc {" $fname`;
   }
   
   foreach $bsc (@allbscs)
   {
      # Extract the first bscID and then return that
      $bsc =~ /bsc {(\d+)\s+\d+}/ && ( $return_value = $1 );
      if ( $return_value ne "" )
      {
         last;
      }
   }
   return $return_value;
}

#-------------------------------------------------------------------------
sub decode_R80_data {
#-------------------------------------------------------------------------

  my ( $tmp_MSCname, $bsc );
  my @return_val;
  my @allbscs;
  my $return_value;
  my @xml_tag;
  my @bscArr;
  my $fname;
   if ($filename =~ /.gz$/ ){
    `mkdir $PROSPECT_HOME/../ftpIN/SIEMENSBSCXML/in/temp_$$`;
    `cp $filename $PROSPECT_HOME/../ftpIN/SIEMENSBSCXML/in/temp_$$`;
    `gunzip $PROSPECT_HOME/../ftpIN/SIEMENSBSCXML/in/temp_$$/*.gz`; 
    $fname = substr($filename, 0, length($filename)-3);
    @allbscs=`grep "BSS-" $PROSPECT_HOME/../ftpIN/SIEMENSBSCXML/in/temp_$$/*`;
    `rm -rf $PROSPECT_HOME/../ftpIN/SIEMENSBSCXML/in/temp_$$`;
   } else {
    $fname = $filename;
    @allbscs=`grep "BSS-" $fname`;
   }
  foreach $bsc (@allbscs)
  {
  @xml_tag = split("-",$bsc);
  @bscArr =split("\"",$xml_tag[1]);
  $return_value = $bscArr[0]; 
     if ( $return_value ne "")
     {
        last;
     }
  }
  return $return_value;
}
# Check that the element_list is correct and exists.
#------------------------------------------------------------------------------
sub CheckList()
#------------------------------------------------------------------------------
{
   if ( ! -r $ElementList)
   {
      terminalError( "Error : Cannot read $ElementList" );
   }
}

# Check that the r55 db/fm mapping files exist.
#------------------------------------------------------------------------------
sub CheckR55Files()
#------------------------------------------------------------------------------
{
   if ( ! -r $R55DBMapping)
   {
      terminalError( "Error : Cannot read $R55DBMapping" );
   }
   if ( ! -r $R55fmMapping)
   {
      terminalError( "Error : Cannot read $R55fmMapping" );
   }
   
}

# Make sure the file exists and is not empty      
#------------------------------------------------------------------------------
sub CheckFile()
#------------------------------------------------------------------------------
{
   if ((! -e $filename) || (-z $filename))
   {
      terminalError("The file $filename is either non-existent or empty");
   }
}

# Split the filename into its separate parts.
#------------------------------------------------------------------------------
sub SplitFileName($)
#------------------------------------------------------------------------------
{
   my ($path, $prefix, $return_val);
   my $fname = shift;
   $fname = `basename $fname`;
   if ($fname =~ /.tar.gz$/ ) {
     $fname = substr($fname, 0, length($fname)-7);
   } elsif ( $fname =~ /.tar$/ ) {
     $fname = substr($fname, 0, length($fname)-4);
   } elsif ($fname =~ /.gz$/ ){
     $fname= substr($fname, 0, length($fname)-3);
   }
   $fname=~s/\.$//;
   my @fnameArr;
   my @fnameArr1;

   if ($fname =~ /.exp$/ )
   {
       ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*exp$');
       @fnameArr = split("\\_", $shortFile);
       # Don't expect any data < 2000 and > 2099
       $time_stamp = "20".substr($fnameArr[2],4,2).substr($fnameArr[2],2,2).substr($fnameArr[2],0,2).substr($fnameArr[2],6,6);
       $return_val = decode_R55_data();

       undef @fnameArr;
       @fnameArr = split("-",$return_val);
       $mscid = $fnameArr[0];
       $nename = $fnameArr[0];
       $bscid = $fnameArr[1];

   } elsif ($fname =~ /.ASCII$/i ){
       if($fname=~/(.*)\:\d+\.\d+\.ascii$/i){
         $baseEntity = getNEType($1);
       }
       if($baseEntity eq $SieCSBaseEntity){
         ($shortFile, $path, $filetype)=fileparse($fname, '\.ascii$');
         @fnameArr = split("\\:", $shortFile);
         $nename    = $fnameArr[0];
         @fnameArr = split("\\.", $fnameArr[1]);
         $time_stamp = $fnameArr[0];
         $filetype = $SieCSBaseEntity.uc($filetype);

       }
       else{
         ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*ASCII$');
         if ($shortFile=~ /\./){
           @fnameArr1 = split("\\.", $shortFile);
           @fnameArr = split("\\_", $fnameArr1[0]); 
           @fnameArr = (@fnameArr,$fnameArr1[1]);
         } 
         else {
           @fnameArr = split("\\_", $shortFile);
         }
         $mscid     = $fnameArr[1];
         $bscid     = $fnameArr[3];
         $nename    = $fnameArr[0];
         $time_stamp = $fnameArr[4];
      }
   }
   elsif ($fname =~ /.xml$/i ){               #change point
       my $oldbaseEntity = $baseEntity;
       if($fname=~/(.*)\.\d+\.\d+\.xml$/i){
         $baseEntity = getNEType($1);
       }

       if($baseEntity eq $SieCSBaseEntity){
         ($shortFile, $path, $filetype)=fileparse($fname, '\.xml$');
         @fnameArr = split("\\.", $shortFile);
         $nename    = $fnameArr[0];
         $time_stamp = $fnameArr[1].$fnameArr[2];
         $filetype = $SieCSBaseEntity.$filetype;

       }
       else{
         $baseEntity = $oldbaseEntity;
         ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*xml$');
         if ($shortFile=~ /\./){
           @fnameArr1 = split("\\.", $shortFile);
           @fnameArr = split("\\_", $fnameArr1[0]);
           @fnameArr = (@fnameArr,$fnameArr1[1]);

         }
         else {
           @fnameArr = split("\\_", $shortFile);
         }
         $mscid     = $fnameArr[1];
         $bscid     = $fnameArr[3];
         $nename    = $fnameArr[0];
         $time_stamp = $fnameArr[4];

      }

   } elsif ( $fname =~ /.RNCpm$/ || $fname =~ /.NODEBpm$/) {
      if ( $fname =~ /.RNCpm$/ ) {
        ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*RNCpm$');
      } else {
        ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*NODEBpm$');
      }
      @fnameArr = split ( '\.', $shortFile ) ;
      $nename = $fnameArr[0];
      $rncid = $fnameArr[1];
      $rncid =~ s/^0+//;
      $time_stamp= $fnameArr[2] . $fnameArr[3] . "00";
   } elsif ( $fname =~ /.CFGpm$/ ) {
      ($shortFile, $path, $filetype)=fileparse($fname, '\.[A-Za-z]*CFGpm$');
      @fnameArr = split ( '\.', $shortFile ) ;
      $nename = $fnameArr[0];
      $rncid = $fnameArr[1];
      $rncid =~ s/^0+//;
      $time_stamp= $fnameArr[2] . "000000";
   } elsif ($fname =~ /\.RHDAT$/ ) {
     terminalError( "File decode does not support RH data files");
   } elsif ($fname =~ /\.MRDAT$/ ) {
     terminalError( "File decode does not support MR data files");
   } elsif ($fname =~ /\.asc$/ ) {
     exit(0);
   } 

   if ( ((!defined $mscid || !defined $bscid) && !defined $rncid && ($baseEntity ne $SieCSBaseEntity))
       || !defined $nename || !defined $time_stamp )
   {
       terminalError ( "Missing field(s) in file name : $fname" );
   }
}

# Get the tstamp, decoding the values
#------------------------------------------------------------------------------
sub GetTstamp()
#------------------------------------------------------------------------------
{

    # Make the timestamp 'looks' valid
    if ($time_stamp =~ /\d+/)
    {
        ConvertDate($time_stamp);
    }
    else
    {
        terminalError("The time stamp field $time_stamp in $filename should be a number");
    }
   
}

# Get the filetype information
#------------------------------------------------------------------------------
sub GetFileTypeInfo()
#------------------------------------------------------------------------------
{
   my $filetype = $_[0];

   my $query = $dbh->prepare("select entity_name from wm_file_control where file_type = ?");

   unless (defined $query) {
     terminalError("ORACLE : Cannot prepare select entity_name query : $DBI::errstr");
   }

   unless ( $query->execute( $filetype ) ) {
     terminalError("ORACLE : Failed to execute select entity_name query : $DBI::errstr");
   }

   $query->bind_columns(\$baseEntity);

   if ( ! $query->fetch) {
     terminalError("Unable to identify file type : $filetype");
   }

}

#SieCS
#to get NE_Type based on the NE Name
sub getNEType
{
  my ($NE_name) = @_;
  open IN, "<$ElementList" or die "Couldn't open element_list";
    while ( my $line = <IN> ) {
      chomp($line);
      if ( $line =~ /^${NE_name}\t/ ) {
        my @fields=split('\t',$line);
        if ( $fields[0] eq $NE_name) {
          return $fields[5];
        }
      }
    }
    close IN;
}

#------------------------------------------------------------------------------
sub connect_to_oracle
#------------------------------------------------------------------------------
{
  unless (exists $ENV{DBSID}) {
    terminalError("ORACLE : Environemt variable DBSIB not set");
  }
  unless (exists $ENV{DBUSER}) {
    terminalError("ORACLE : Environemt variable DBUSER not set");
  }
  unless (exists $ENV{DBPASSWD}) {
    terminalError("ORACLE : Environemt variable DBPASSWD not set");
  }

  my $data_source = "dbi:Oracle:$ENV{DBSID}";
  my $dbh;

  unless ($dbh = DBI->connect($data_source, $ENV{DBUSER}, $ENV{DBPASSWD},
                              { RaiseError => 0,
                                PrintError => 0,
                                AutoCommit => 1 } ) ) {
    terminalError("ORACLE : Cannot open connection to database:  $ENV{DBUSER}/$ENV{DBPASSWD}\@$ENV{DBSID}\n$DBI::errstr");
  }

  return $dbh;
}

