#!/bin/ksh
#> This script will install SieUMTS patch for SieGSM
#> patch: 4.0.4.0.10
#

# variables used after <productVersion> and <wsVersion> drop.
export NEPATCH_START_DIR=`pwd`
export ARCHIVE_DIR=$NEPATCH_START_DIR/Archive
export BACKUP_DIR=$NEPATCH_START_DIR/Backup

########################################################
# Check system values required for this module but not checked by the
# main installer.
if [[ -z $FLEXPM_HOME ]]; then
  echo "+> FLEXPM_HOME environment is not defined"
  echo "+> Please ensure that this is defined and then reinstall upgrade"
  echo ""
  exit 2
fi

if [[ ! -d $FLEXPM_HOME ]]; then
  echo "+> FLEXPM_HOME environment value '$FLEXPM_HOME'"
  echo "+> is not a valid directory. Please check this value"
  echo ""
  exit 2
fi

########################################################
# Check system values required for this module but not checked by the
# main installer.
if [[ -z $SieGSM_HOME ]]; then
  echo "+> SieGSM_HOME environment is not defined"
  echo "+> Please ensure that this is defined and then reinstall upgrade"
  echo ""
  exit 2
fi

if [[ ! -d $SieGSM_HOME ]]; then
  echo "+> SieGSM_HOME environment value '$SieGSM_HOME'"
  echo "+> is not a valid directory. Please check this value"
  echo ""
  exit 2
fi

# Begin the upgrade
echo "+ Installing SieGSM component upgrades ... "
date +"+ Component upgrade is started at %T, %D ... "

cd ${NEPATCH_START_DIR}
if [ ! -d $BACKUP_DIR ]
then
  mkdir $BACKUP_DIR
fi

echo ""
echo "+ Backing up original SieGSM module files ..."
cd $SieGSM_HOME
for file in `gtar -tzf ${NEPATCH_START_DIR}/SieGSM_1.0.tgz`
do
  if [ -f $file ]
  then
    if [ ! -d ${ARCHIVE_DIR}/`dirname $file` ]
    then
      mkdir -p ${ARCHIVE_DIR}/`dirname $file`
    fi
    cp $file ${ARCHIVE_DIR}/`dirname $file`
    chmod +w $file
  fi
done

cd $NEPATCH_START_DIR/Archive
gtar -czf ../Module_Archive_`date '+%y%m%d_%H%M%S'`.tar.gz *
cd $NEPATCH_START_DIR
rm -rf ${ARCHIVE_DIR}
mv -f Module_Archive*.tar.gz $BACKUP_DIR/
cd $SieGSM_HOME

echo "+ Installing new/updated files for SieGSM module ..."
gtar -xvzf ${NEPATCH_START_DIR}/SieGSM_1.0.tgz
for file in `gtar -tzf ${NEPATCH_START_DIR}/SieGSM_1.0.tgz`
do
	chmod +x $file
done
date +"+ SieGSM component upgrade is completed at %T, %D ....."
echo ""
exit 0
$
